/*******************************************************************************
 *                              StgSvcsMethods.hpp
 *
 * Stager catalog services grouped by category. This file is not compilable
 * and it is provided for documentation purposes only.
 *
 * @author Giuseppe Lo Presti
 *******************************************************************************/


//------------------------------------------------------------------------------
// ICommonSvc
//------------------------------------------------------------------------------
stager::SvcClass*               selectSvcClass           (const string name)   // everywhere
stager::FileClass*              selectFileClass          (const string name)      // db_svc & hsmtools
stager::FileSystem*             selectFileSystem         (const string mountPoint, const string diskServer)  // FS & Job
stager::Tape*                   selectTape               (string vid, int side, int tpmode)   // VdqmSvc & rtcopy 


//------------------------------------------------------------------------------
// ITapeSvc -> rtcopy
//------------------------------------------------------------------------------
vector<stager::Tape*>           tapesToDo()
vector<stager::Stream*>         streamsToDo()

vector<stager::Segment*>        segmentsForTape          (stager::Tape* searchItem)
int                             anySegmentsForTape       (stager::Tape* searchItem)

stager::DiskCopyForRecall*      bestFileSystemForSegment (stager::Segment *segment)
stager::TapeCopyForMigration*   bestTapeCopyForStream    (stager::Stream* searchItem)
bool                            anyTapeCopyForStream     (stager::Stream* searchItem)

void                            fileRecalled             (stager::TapeCopy* tapeCopy)
void                            fileRecallFailed         (stager::TapeCopy* tapeCopy)
void                            resetStream              (stager::Stream* stream)
vector<stager::Segment*>        failedSegments()

void                            streamsForTapePool       (stager::TapePool* tapePool)    // MigHunter
vector<stager::TapeCopy*>       selectTapeCopiesForMigration(stager::SvcClass *svcClass)  // MigHunter


//------------------------------------------------------------------------------
// IStagerSvc -> scheduler, stager, error
//------------------------------------------------------------------------------
stager::CastorFile*             selectCastorFile         (u_signed64 fileId, string nsHost, u_signed64 svcClass, u_signed64 fileClass, u_signed64 fileSize)
stager::DiskCopyForRecall*      recreateCastorFile       (stager::CastorFile *castorFile, stager::SubRequest *subreq)

stager::SubRequest*             subRequestToDo           (vector<ObjectsIds> &types)
void                            archiveSubReq            (_signed64 subReqId)
stager::SubRequest*             subRequestFailedToDo()                                     // errorSvc
bool                            updateAndCheckSubRequest (stager::SubRequest *subreq)       // errorSvc
bool                            isSubRequestToSchedule   (stager::SubRequest* subreq, list<stager::DiskCopyForRecall*>& sources)

void                            updateFileSystemForJob   (string fileSystem, string diskServer, u_signed64 fileSize)    // scheduler
void                            bestFileSystemForJob     (char** fileSystems, char** machines, u_signed64* minFree, int fileSystemsNb, string* mountPoint, string* diskServer)

void                            stageRelease             (const u_signed64 fileId, const string nsHost)
void                            stageRm                  (const u_signed64 fileId, const string nsHost)


//------------------------------------------------------------------------------
// IFSSvc -> hsmtools, fs_service
//------------------------------------------------------------------------------
stager::DiskPool*               selectDiskPool           (const string name)
stager::TapePool*               selectTapePool           (const string name)
stager::DiskServer*             selectDiskServer         (const string name)


//------------------------------------------------------------------------------
// IJobSvc
//------------------------------------------------------------------------------
// job/stagerJob.c needs a selectDiskCopy() method?
stager::Request*                requestToDo              ()
stager::DiskCopy*               getUpdateStart           (stager::SubRequest* subreq, stager::FileSystem* fileSystem, list<stager::DiskCopyForRecall*>& sources, bool* emptyFile)
stager::DiskCopy*               putStart                 (stager::SubRequest* subreq, stager::FileSystem* fileSystem)
stager::DiskCopy*               putDoneStart             (u_signed64 subreqId)

void                            prepareForMigration      (stager::SubRequest* subreq, u_signed64 fileSize)

void                            disk2DiskCopyDone        (u_signed64 diskCopyId, stager::DiskCopyStatusCodes status)
void                            getUpdateDone            (u_signed64 subReqId)

void                            getUpdateFailed          (u_signed64 subReqId)
void                            putFailed                (u_signed64 subReqId)


//------------------------------------------------------------------------------
// IGCSvc
//------------------------------------------------------------------------------
stager::Request*                requestToDo              ()
vector<stager::GCLocalFile*>*   selectFiles2Delete       (string diskServer)
void                            filesDeleted             (vector<u_signed64*>& diskCopyIds)
void                            filesDeletionFailed      (vector<u_signed64*>& diskCopyIds)


//------------------------------------------------------------------------------
// IQuerySvc
//------------------------------------------------------------------------------
stager::Request*                requestToDo              ()
list<stager::DiskCopyInfo>      diskCopies4File          (string fileId, string nsHost, u_signed64 svcClassId)
list<stager::DiskCopyInfo>      diskCopies4Request       (stager::RequestQueryType reqType, string param, u_signed64 svcClassId)


//------------------------------------------------------------------------------
// IVdqmSvc
//------------------------------------------------------------------------------
TapeServer*                     selectTapeServer         (const std::string serverName)
int                             checkTapeRequest         (const TapeRequest *tapeRequest)
void                            matchTape2TapeDrive      (TapeDrive** freeTapeDrive, TapeRequest** waitingTapeRequest)
TapeAccessSpecification*        selectTapeAccessSpecification(const int accessMode, const string density, const string tapeModel
DeviceGroupName*                selectDeviceGroupName    (const string dgName)
std::vector<TapeRequest*>*      selectTapeRequestQueue   (const string dgn,	const string requestedSrv)
std::vector<TapeDrive*>*        selectTapeDriveQueue     (const string dgn, const string requestedSrv)
TapeDrive*                      selectTapeDrive          (const newVdqmDrvReq_t* driveRequest, TapeServer* tapeServer)

bool                            existTapeDriveWithTapeInUse(const string volid)
bool                            existTapeDriveWithTapeMounted(const string volid)
castor::stager::Tape*           selectTapeByVid          (const string vid)
TapeRequest*                    selectTapeReqForMountedTape(const TapeDrive* tapeDrive)

