#!/usr/bin/python

devs = ['Sebastien', 'Giuseppe', 'Giulia', 'Rosa', 'Felix', 'Shaun', 'Dennis', 'Viktor', 'Miroslav']
priorities = {'Low' : 0, 'Medium' : 1, 'High' : 2, 'N/A' : 3}
colors = {'Low' : '#00aa00', 'Medium' : '#0000aa', 'High' : '#aa0000', 'N/A' : '#aaffaa', 'Header' : '#ffffff'}
durations = {'day' : 1, 'days' : 1, 'week' : 7, 'weeks' : 7,
             'month' : 30, 'months' : 30}
months = {'January' : 1, 'February' : 2, 'March' : 3, 'April' : 4,
          'May' : 5, 'June' : 6, 'July' : 7, 'August' : 8,
          'September' : 9, 'October' : 10, 'November' : 11, 'December' : 12}
allocations = {}
currentLine = ''
sectionNb = 0
subsectionNb = 0
ignoreHistory = 1

def getline():
  global currentLine
  currentLine = input.readline()
  if currentLine == '':
    raise ValueError, 'EOF'

def getItem(first = 0):
  global currentLine
  if currentLine == '' :
    getline()
  # go to next sectoin or subsection or item
  while(not currentLine.startswith('\item[')): getline()
  # item title
  title = currentLine[currentLine.find('[')+1:currentLine.find(']')]
  content = currentLine[currentLine.find(']')+1:len(currentLine)].replace('\_','_')
  getline()
  while(not currentLine.startswith('\\')):
    content = content + ' ' + currentLine.replace('\_','_')
    getline()
  return (title, content[0:len(content)-1])

def writeResources():
  output.write("    <resources>\n");
  i = 0
  for d in devs:
    output.write('        <resource id = "' + str(i) +
                 '" name="' + d + '" function="SoftwareDevelopment:2" contacts="" phone="" />\n')
    i = i  + 1
  output.write("    </resources>\n\n");

def parseManPower(manpower):
  res = []
  manpowers = manpower.split(';')
  tot = 0.0
  for m in manpowers:
    if m.count(',') > 0:
      (who,sload) = m.split(',')
      who = who.strip()
      sload = sload.replace('\\%','')
      load = float(sload)/100
    else:
      who = m.strip()
      load = 1.0
    tot = tot + load
    res.append([who, load])
  return res, tot  

def writeAllocations():
  output.write("    <allocations>\n");
  for (tid, manpower) in allocations.items():
    for m, l in manpower:
      if devs.count(m) > 0:
        output.write('        <allocation task-id="' +
                     str(tid) + '" resource-id="' +
                     str(devs.index(m)) +
                     '" function="Default:2" responsible="true" ' +
                     'load="' + str(int(100*l)) + '"/>\n')
  output.write("    </allocations>\n\n");

def writeVacations():
  output.write("    <vacations>\n");
#  output.write('        <vacation start="2006-05-22" end="2006-06-04" resourceid="2"/>\n')
#  output.write('        <vacation start="2006-06-22" end="2006-07-02" resourceid="0"/>\n')
#  output.write('        <vacation start="2006-08-15" end="2006-08-30" resourceid="0"/>\n')
#  output.write('        <vacation start="2006-12-01" end="2007-01-07" resourceid="7"/>\n')
  output.write("    </vacations>\n\n");  

def txt2Days(duration, perc):
  if duration == 'Ongoing':
    result = 1400 # 4 years
  else:
    (nb, unit) = duration.split()
    result = int(nb) * durations[unit]
    result = result / perc
    result = result * 1.66   # take holidays into account
  return result

def txt2Date(date):
  result = '2008-'
  if date.count('2007') > 0: result = '2007-'
  if date.count('2006') > 0: result = '2006-'
  if date.count('2005') > 0: result = '2005-'
  if date.count('quarter') > 0:
    if date.startswith('1'):
      result = result + '01-01'
    elif date.startswith('2'):
      result = result + '04-01'
    elif date.startswith('3'):
      result = result + '07-01'
    else:
      result = result + '10-01'
  else:
    (day, month, year) = date.split()
    day = day.replace('$^{st}$', '').replace('$^{nd}$', '').replace('$^{rd}$', '').replace('$^{th}$', '')
    result = result + str(months[month]) + '-' + day
  return result

input = open('Tasks.tex')
output = open('Tasks.gan', 'w')

output.write(open('Tasks.gan.header').read())
tid = 0;

firstSection = 1;
# loop on tasks
try:
  while(1):
    getline()
    # go to next subsection
    ok = 0
    while (not ok):
      if (currentLine.startswith('\subsection{')) :
        subsectionNb = subsectionNb + 1
        ok = 1
      elif (currentLine.startswith('\section{')):
        sectionNb = sectionNb + 1
        subsectionNb = 0
        title = currentLine[currentLine.find('{')+1:currentLine.find('}')].replace('\_','_')
        if ignoreHistory and title == "Completed":
          raise ValueError
        if firstSection == 0:
          output.write('        </task>\n')
        else:
          firstSection = 0
        output.write('        <task id="' + str(tid) +
                     '" name="' + title + '" color="' + colors["Header"] +
                     '" meeting="false" expand="true">\n')
        tid = tid + 1
        getline()
      elif (currentLine.startswith('\setcounter{section}')):
        start = currentLine.find('}{') + 2
        end = currentLine.find('}', start)
        sectionNb = int(currentLine[start:end])
        getline()
      elif (currentLine.startswith('\setcounter{subsection}')):
        start = currentLine.find('}{') + 2
        end = currentLine.find('}', start)
        subsectionNb = int(currentLine[start:end])
        getline()
      else:
        getline()
    # tasks title
    title = currentLine[currentLine.find('{')+1:currentLine.find('}')].replace('\_','_')
    currentLine = ''
    (head, desc) = getItem()
    (head, priority) = getItem()
    (head, smanpower) = getItem()
    manpower, totalPerc = parseManPower(smanpower)
    (head, duration) = getItem()
    (head, date) = getItem()
    (head, completion) = getItem()
    if (head != 'Completion'):
      completion = '0'
    elif 'N/A' == completion:
      completion = '0'
    else:
      completion = completion.replace('\\%', '')
    output.write('            <task id="' + str(tid) +
                 '" name="T' + str(sectionNb) + '.' +
                 str(subsectionNb) + ' : ' + title + '" color="' +
                 colors[priority] +
                 '" meeting="false" start="' +
                 txt2Date(date) +
                 '" duration="' +
                 str(int(txt2Days(duration, totalPerc))) +
                 '" complete="' + completion +
                 '" priority="' +
                 str(priorities[priority]) +
                 '" expand="true">\n')
    output.write('                <notes>\n')
    output.write('                ' + desc + '\n')
    output.write('                </notes>\n')
    output.write('            </task>\n')
    allocations[tid] = manpower
    tid = tid + 1
except ValueError:
  output.write("        </task>\n\n");
  output.write("    </tasks>\n\n");
  writeResources()
  writeAllocations()
  writeVacations()
  output.write(open('Tasks.gan.trailer').read())
