.TH draindiskserver 1 "$Date: 2009/02/12 11:02:59 $" CASTOR "Draining Diskserver Commands"
.SH NAME
draindiskserver \- management of draining filesystems and diskservers

.SH SYNOPSIS
.B draindiskserver
[\fIOPTIONS\fR]...

.SH DESCRIPTION
.B draindiskserver
is an administration command that allows for the draining of filesystems and
diskservers in an automated way.

.SH OPTIONS
Supported operations:
.TP
.B -a,\ \-\-add
Add a new filesystem or diskserver to the list of those to be drained.
.TP
.B \-\-restart
Restart the draining process of a given filesystem or diskserver with the
previously defined options.
.TP
.B -d,\ \-\-delete
Delete/Cancel the draining process of a given filesystem or diskserver.
.TP
.B -q,\ \-\-query
Query the filesystems currently being drained.

.TP
Add options (with \fB\-a\fR or \fB\-\-add\fR):
.TP
.B -S,\ \-\-svcclass=\fINAME\fR
The name of the service class that files should be replicated too. This option
can be omitted if the filesystems to be added belong to a diskpool which is
linked to only one service class.
.TP
.B -t,\ \-\-tansfers=\fINUM\fR
The maximum number of transfers allowed to run concurrently per filesystem.
(Default: 50). The value must be greater than 0. It is recommended when setting
this value to set more transfers then job slots in LSF. This compensates for any
processing overhead that the diskserver draining logic introduces and makes sure
that there are always transfers pending in the LSF queue.
.TP
.B --auto-delete
Delete files automatically after replication. (Default: NO). Files deleted as a
resulted of the draining process will have the GcType: 'Draining filesystem'.
.TP
.B --file-mask=[\fISTAGED|CANBEMIGR|ALL\fR]
Mask used to decide which files need to be replicated. (Default: CANBEMIGR)

.TP
Query options (with \fB\-q\fR or \fB\-\-query\fR):
.TP
.B -c,\ \-\-config
List the configuration options.
.TP
.B -f,\ \-\-failures
List the files which failed to be replicated and if available the reason why.
.TP

\fBNote:\fR  If no additional \fB\-\-query\fR options are specified, the default
behaviour is to display a summary of the draining process for all registered
filesystems. The output of the summary contains the following information
grouped by diskserver:
.RS
.TP
.B MountPoint
The name of the mountpoint/filesystem.
.TP
.B Created
The creation time of the request to drain the filesystem in the format:
\fB%d-%b-%Y %T\fR (see, `man strftime`).
.TP
.B TFiles
The total number of files to be replicated.
.TP
.B TSize
The total size of the files to replicated expressed in a human readable format
(e.g, 1K, 234M 2G) using powers of 1000 (SI Units).
.TP
.B RFiles
The remaining number of files to be replicated.
.TP
.B RSize
The remaining size of the files to be replicated expressed in a human readable
format (e.g, 1K, 234M 2G) using powers of 1000 (SI Units).
.TP
.B Failed
The number of files which have failed to be replicated. The exact files which
failed can be listed using the \fB\-\-query\fR option.
.TP
.B RunTime
An interval format string representing the number of days, hours, minutes and
seconds that the draining process has been running for the filesystem.
.TP
.B Progress
The progress of the draining process represented as a percentage of the size of
the data transferred.
.TP
.B ETC
\fBE\fRstimated \fBT\fRime \fBC\fRompletion represented as an interval format
string showing the number of estimated days, hours, minutes and seconds left.
Note: N/A will be displayed until at least 10% of the data has been replicated.
.TP
.B Status
The status of the draining process. One of: \fBCREATED\fR, \fBINITIALIZING\fR,
\fBRUNNING\fR, \fBSTALLED\fR, \fBINTERRUPTED\fR, \fBFAILED\fR, \fBCOMPLETED\fR,
\fBDELETING\fR and \fBRESTART\fR
.RE

.TP
Common options for all operations:
.TP
.B -n,\ \-\-nodename=\fINODENAME\fR
The name of the diskserver to perform the operation on.
.TP
.B -m,\ \-\-mountpoint=\fIMOUNTPOINT\fR
The name of the mountpoint to perform the operation on. Note: if the mountpoint
option is omitted, the operation will apply to all filesystems associated with
the diskserver.
.TP
.B --script
Display the output using a style suitable for parsing by scripts.
.TP
.B --help
Display this help and exit

.SH NOTES
When querying filesystems:

.RS
The \fBINTERRUPTED\fR status is displayed when a filesystem that was being
drained was stopped due to a change of the diskserver or filesystem status which
meant that the filesystem itself was no longer in a \fBDRAINING\fR state. To fix
this, the filesystem must be placed into a \fBDRAINING\fR state using the
`\fBrmAdminNode\fR` command and the filesystem resubmitted for draining.

The \fBSTALLED\fR status is displayed when a filesystem that was being drained
has not processed any files within the last 60 minutes.

Filesystems that are in a \fBCOMPLETED\fR status will be automatically removed
after 7 days or on an explicit call to delete the filesystem using the
`\fBrmAdminNode\fR` command with the \fB-d\fR option.
.RE

The processing of filesystems:

.RS
All filesystems in a \fBRUNNING\fR status are processed in parallel and the order
in which files are selected for replication is dependent on the status of the
file and its age. Therefore, the files in a \fBCANBEMIGR\fR state will be
replicated first, followed by \fBSTAGED\fR files.

If a file being processed, already has an outstanding request to replicate it
(e.g. a stager_get initiated by a user) the draining logic is instructed to wait
for that transfer to complete and not start a second transfer.
.RE

.SH FILES
The
.B draindiskserver
tool requires SQL access to the STAGER database and therefore users executing
this command must be able to read the \fB/etc/castor/ORASTAGERCONFIG\fR password
file.

.SH EXIT STATUS
This program returns 0 if the operation was successful or > 0 if the operation
failed.

.SH SEE ALSO
.BR rmAdminNode(1),
.BR diskServer_qry(1)

.SH AUTHOR
\fBCASTOR\fP Team <castor.support@cern.ch>
