.\" @(#)$RCSfile: enterSvcClass.man,v $ $Revision: 1.13 $ $Date: 2009/07/23 12:22:03 $ CERN IT/ADC Olof Barring
.\" Copyright (C) 2005 by CERN IT/ADC
.\" All rights reserved
.\"
.TH ENTERSVCCLASS 1 "$Date: 2009/07/23 12:22:03 $" CASTOR "stager catalogue administrative commands"
.SH NAME
enterSvcClass \- define a new service class in the stager catalogue
.SH SYNOPSIS
.B enterSvcClass
[
.BI -h,
.BI --help
]
.BI --Name " name"
[
.BI --NbDrives " nbDrives"
]
[
.BI --DefaultFileSize " defaultFileSize"
]
[
.BI --MaxReplicaNb " maxReplicaNb"
]
[
.BI --FailJobsWhenNoSpace " [yes|no]"
]
[
.BI --Disk1Behavior " [yes|no]"
]
[
.BI --ForcedFileClass " forcedFileClass"
]
[
.BI --ReplicateOnClose " [yes|no]"
]
[
.BI --GcPolicy " gcPolicyName"
]
[
.BI --MigratorPolicy " migratorPolicy"
]
[
.BI --StreamPolicy " streamPolicy"
]
[
.BI --RecallerPolicy " recallerPolicy"
]
[
.BI --TapePools " tapePool1:tapePool2:..."
]
[
.BI --DiskPools " diskPool1:diskPool2:..."
]
.SH DESCRIPTION
.B enterSvcClass
enters a new service class definition in the CASTOR stager catalogue
.LP
This command requires database client access to the stager catalogue
.TP
.BI \-h,
.BI \-\-help
Get usage information
.TP
.BI \-\-Name " name"
is the name of the new service class. An error is printed and the command
fails if an existing service class with the same name is found
.TP
.BI \-\-NbDrives " nbDrives"
The number of tape drives this service class is allowed to use concurrently
for migration. There is no service class attribute to control the number tape drives used
for tape recall.
.TP
.BI \-\-DefaultFileSize " defaultFileSize"
The default file size allocation for
.IR stagePut
requests not specifying any filesize.
.TP
.BI \-\-MaxReplicaNb " maxReplicaNb"
Specifies the maximum number of disk resident replicas for a given CASTOR file
allowed for files recalled or created in this service class. Note that specifying
zero (0) replica, allows for an umlimited number of replicas (not recommended
unless a replication policy is specified, see below). If this option is not
specified or if a negative number is given, the default of one (1) replica is
applied.
.TP
.BI \-\-FailJobsWhenNoSpace " [yes|no]"
Defines whether or not jobs requesting new space are failed when the pool(s) connected
with this service class is(are) full. The supported values are yes or no, and the default
value is yes.
.TP
.BI \-\-Disk1Behavior " [yes|no]"
Defines whether or not this service class has Disk1 behavior, which means that garbage
collection is disabled and the flag FailJobsWhenNoSpace is set to yes.
The supported values are yes or no.
.TP
.BI \-\-ForcedFileClass " forcedFileClass"
Specifies the name of the file class to force on files written to this service class. The
forced file class overrides the file class that the user specifies. This parameter allows
specifying for this service class any TapeN retention policy, where N is the number of
tape copies of the forced file class.
. If an empty string ("") is provided, the file class that the file was written to will be used.
.TP
.BI \-\-ReplicateOnClose " [yes|no]"
Defines whether or not replication on close is enabled for this service class. The
supported values are yes or no. When enabled, once a file is created and finalized/closed it
will be replicated so that MaxReplicaNb copies exist in the same pool where the file resides.
Note: this functionality will not rebalance the number copies in the pool in response to
diskservers which are no longer in production!
.TP
.BI \-\-GcPolicy " gcPolicyName"
Specifies the name of the garbage collection policy to be used for this service class.
This value is irrelevant if Disk1Behavior is set to yes.
.TP
.BI \-\-MigratorPolicy " migratorPolicy"
Specifies the name of the migrator policy to be used for this service
class. The policy should either be an executable external script known to the CASTOR
expert system (see the
.BI expertd(8castor)
man-page).
.TP
.BI \-\-StreamPolicy " streamPolicy"
Specifies the name of the stream policy to be used for this service class. The policy
will be used to determine the number of streams that should be running based on the
number of files and volume of data that should be written too tape. If an empty string ("")
is provided the existing policy setting (if any) will be revoked
.TP
.TP
.BI \-\-RecallerPolicy " recallerPolicy"
Specifies the name of the recaller policy to be used for this service
class. The policy should either be an executable external script known to the CASTOR
expert system (see the
.BI expertd(8castor)
man-page).
.TP
.BI \-\-TapePools " tapePool1:tapePool2:..."
Lists the names of the tape pools that this service class is allowed to use. If
any of the specified tape pools does not exist in the stager catalog, it will be
created and linked to the service class. If more than one tape pool is specified,
the names should be separated by colon (:).
.TP
.BI \-\-DiskPools " diskPool1:diskPool2:..."
Lists the names of the disk pools that this service class is allowed to use. If
any of the specified disk pools does not exist in the stager catalog, it will be
created and linked to the service class. If more than one disk pool is specified,
the names should be separated by colon (:).
.LP
A service class groups together the resources and the storage attributes and policies
governing the disk resident files. The resources are composed by the disk pools
that the service class should use for client user requests for creating new files
as well as requests for files that need to be recalled from tape. The tape pools
are only relevant for the tape migration.

The SvcClass together with the FileClass attributes defined in the stager catalogue
are used to determine, for instance, how many copies and which tape pools should
be used when migrating the file to tape (see
.B NOTES
section below).
.SH EXIT STATUS
This program returns 0 if the operation was successful or >0 if the operation
failed.
.SH EXAMPLES
.nf
.ft CW
enterSvcClass --Name public --NbDrives 2 --TapePools user:largeuser --DiskPools default

enterSvcClass --Name alicemdc6 --NbDrives 24 --DefaultFileSize 3221225472 --TapePools alicemdc6_513:alicemdc6_613 --DiskPools default
.ft
.fi
.SH SEE ALSO
.BR modifySvcClass(1castor) ,
.BR deleteSvcClass(1castor) ,
.BR printSvcClass(1castor) ,
.BR expertd(8castor),
.BR adminMultiInstance(1castor)
.SH NOTES
The definition of FileClass and SvcClass classes in the stager catalogue is
an important change with the new CASTOR stager. The previous CASTOR stgdaemon,
took the following important migration parameters from the CASTOR name server:
number of copies on tape, the tape pools to be used and the maximum number of
drives to be used when starting the migration streams. Those attributes are now
defined in the FileClass and SvcClass tables of the stager catalogue. The
basic motivation for this change was to be able to disentangle file attributes
such as number of tape copies from resource attributes such as how many drives
should be used for migration.
.SH AUTHOR
\fBCASTOR\fP Team <castor.support@cern.ch>
