.\" @(#)$RCSfile: Cns_setsegattrs.man,v $ $Revision: 1.6 $ $Date: 2009/07/23 12:22:04 $ CERN IT-PDP/DM Jean-Philippe Baud
.\" Copyright (C) 1999-2002 by CERN/IT/PDP/DM
.\" All rights reserved
.\"
.TH CNS_SETSEGATTRS 3 "$Date: 2009/07/23 12:22:04 $" CASTOR "Cns Library Functions"
.SH NAME
Cns_setsegattrs \- set file segment attributes
.SH SYNOPSIS
.B #include <sys/types.h>
.br
\fB#include "Cns_api.h"\fR
.sp
.BI "int Cns_setsegattrs (const char *" path ,
.BI "struct Cns_fileid *" file_uniqueid ,
.BI "int " nbseg ,
.BI "struct Cns_segattrs *" segattrs,
.BI "time_t " last_mod_time )
.SH DESCRIPTION
.B Cns_setsegattrs
sets the file segment(s) attributes. Takes care that the file was not modified by somebody else in the mean time.
If it was, ignores the request and fails with ENSFILECHG.
This function should be called by the stager to set segment size, tape
visual identifier and file sequence number or blockid when all segments
which belong to a file have been written to tape.
The file can be identified by
.I path
name or by
.IR file_uniqueid .
If both are specified,
.I file_uniqueid
is used.
.TP
.I path
specifies the logical pathname relative to the current CASTOR directory or
the full CASTOR pathname.
.TP
.I nbseg
the number of segments in the segattrs array.
.TP
.I segattrs
is a pointer to an array of Cns_segattrs structures provided by the application.
.PP
.nf
.ft CW
struct Cns_segattrs {
        int             copyno;
        int             fsec;           /* file section number */
        u_signed64      segsize;        /* file section size */
        int             compression;    /* compression factor */
        char            s_status;       /* 'D' --> deleted */
        char            vid[CA_MAXVIDLEN+1];
        int             side;
        int             fseq;           /* file sequence number */
        unsigned char   blockid[4];     /* for positionning with locate */
        char            checksum_name[CA_MAXCKSUMNAMELEN+1];
        unsigned long   checksum;
};
.ft
.fi
.TP
.I copyno
is the copy number. If specified as zero, nsd will compute the actual
value.
.TP
.I fsec
is the file segment number, It is 1 for single volume files and >= 1 for files
spanning over several tapes.
.TP
.I last_mod_time
specifies the last modification time of the file, as known by the caller. This is used to double check that the file was not modified concurrently by somebody else.
.SH RETURN VALUE
This routine returns 0 if the operation was successful or -1 if the operation
failed. In the latter case,
.B serrno
is set appropriately.
.SH ERRORS
.TP 1.3i
.B ENOENT
The referenced file does not exist anymore.
.TP
.B EACCES
Search permission is denied on a component of the
.I path
prefix or the caller effective user ID does not match the owner ID of the file
or write permission on the file itself is denied.
.TP
.B ENSFILECHG
The file has been changed/overwritten by somebody else since the last_mod_time given. Request was thus ignored.
.TP
.B EFAULT
.I path
and
.I file_uniqueid
or
.I vid
are NULL pointers.
.TP
.B ENOTDIR
A component of
.I path
prefix is not a directory.
.TP
.B EISDIR
The file is not a regular file.
.TP
.B EINVAL
The length of
.I vid
exceeds
.BR CA_MAXVIDLEN .
.TP
.B ENAMETOOLONG
The length of
.I path
exceeds
.B CA_MAXPATHLEN
or the length of a
.I path
component exceeds
.BR CA_MAXNAMELEN .
.TP
.B SENOSHOST
Host unknown.
.TP
.B SENOSSERV
Service unknown.
.TP
.B SECOMERR
Communication error.
.TP
.B ENSNACT
Name server is not running or is being shutdown.
.SH SEE ALSO
.BR Castor_limits(4) ,
.BR Cns_chdir(3) ,
.BR Cns_getsegattrs(3)
.SH AUTHOR
\fBCASTOR\fP Team <castor.support@cern.ch>
