/******************************************************************************
 *              stager_2.1.9-4_to_2.1.9-5.sql
 *
 * This file is part of the Castor project.
 * See http://castor.web.cern.ch/castor
 *
 * Copyright (C) 2003  CERN
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * This script upgrades a CASTOR v2.1.9-4 STAGER database to v2.1.9-5
 *
 * @author Castor Dev team, castor-dev@cern.ch
 *****************************************************************************/

/* Stop on errors */
WHENEVER SQLERROR EXIT FAILURE
DECLARE
  unused VARCHAR2(30);
BEGIN
  UPDATE UpgradeLog
     SET failureCount = failureCount + 1
   WHERE schemaVersion = '2_1_9_4'
     AND release = '2_1_9_5'
     AND state != 'COMPLETE';
  COMMIT;
EXCEPTION WHEN NO_DATA_FOUND THEN
  NULL;
END;
/

/* Version cross check and update */
DECLARE
  unused VARCHAR(100);
BEGIN
  SELECT release INTO unused FROM CastorVersion
   WHERE release LIKE '2_1_9_4%';
EXCEPTION WHEN NO_DATA_FOUND THEN
  -- Error, we can't apply this script
  raise_application_error(-20000, 'PL/SQL release mismatch. Please run previous upgrade scripts before this one.');
END;
/

INSERT INTO UpgradeLog (schemaVersion, release, type)
VALUES ('2_1_9_4', '2_1_9_5', 'TRANSPARENT');
COMMIT;


/* Schema changes go here */
/**************************/

/* SQL statements for type TapeGatewayRequest */
CREATE TABLE TapeGatewayRequest (accessMode NUMBER, startTime INTEGER, lastVdqmPingTime INTEGER, vdqmVolReqId NUMBER, nbRetry NUMBER, lastFseq NUMBER, id INTEGER CONSTRAINT PK_TapeGatewayRequest_Id PRIMARY KEY, streamMigration INTEGER, tapeRecall INTEGER, status INTEGER) INITRANS 50 PCTFREE 50 ENABLE ROW MOVEMENT;

/* SQL statements for type TapeGatewaySubRequest */
CREATE TABLE TapeGatewaySubRequest (fseq NUMBER, id INTEGER CONSTRAINT PK_TapeGatewaySubRequest_Id PRIMARY KEY, tapecopy INTEGER, request INTEGER, diskcopy INTEGER) INITRANS 50 PCTFREE 50 ENABLE ROW MOVEMENT;

/* Index and Constraints for the tapegateway tables */
CREATE INDEX I_TGSubRequest_Request ON TapeGatewaySubRequest(request);
CREATE UNIQUE INDEX I_TGSubRequest_TapeCopy ON TapeGatewaySubRequest(tapeCopy);
CREATE UNIQUE INDEX I_TGRequest_Tape ON TapeGatewayRequest(tapeRecall);
CREATE UNIQUE INDEX I_TGRequest_Stream ON TapeGatewayRequest(streamMigration);
CREATE UNIQUE INDEX I_TGRequest_VdqmVolReqId ON TapeGatewayRequest(vdqmVolReqId);

ALTER TABLE TapeGatewaySubRequest ADD CONSTRAINT FK_TGSubRequest_TC FOREIGN KEY (tapeCopy) REFERENCES TapeCopy (id);
ALTER TABLE TapeGatewaySubRequest ADD CONSTRAINT FK_TGSubRequest_DC FOREIGN KEY (diskCopy) REFERENCES DiskCopy (id);
ALTER TABLE TapeGatewaySubRequest ADD CONSTRAINT FK_TGSubRequest_TGR FOREIGN KEY (request) REFERENCES TapeGatewayRequest(id);
ALTER TABLE TapeGatewayRequest ADD CONSTRAINT FK_TGSubRequest_SM FOREIGN KEY (streamMigration) REFERENCES Stream (id);
ALTER TABLE TapeGatewayRequest ADD CONSTRAINT FK_TGSubRequest_TR FOREIGN KEY (tapeRecall) REFERENCES Tape (id);
ALTER TABLE TapeGatewaySubRequest ADD CONSTRAINT UN_TGSubRequest_TR_DC UNIQUE (request, diskcopy);

INSERT INTO CastorConfig 
  VALUES ('tape', 'interfaceDaemon', 'rtcpclientd', 'The name of the daemon used to interface to the tape system');

/* #63347: Improve query execution in the checkFailJobsWhenNoSpace function */
DECLARE
  unused VARCHAR2(30);
BEGIN
  SELECT index_name INTO unused
    FROM user_indexes
   WHERE index_name = 'I_STAGEPUTREQUEST_SVCCLASS';
EXCEPTION WHEN NO_DATA_FOUND THEN
  EXECUTE IMMEDIATE 
    'CREATE INDEX I_StagePutRequest_SvcClass ON StagePutRequest (svcClass)';
END;
/

/* #63362: Database optimisations in storeClusterStatus */
DECLARE
  unused VARCHAR2(30);
BEGIN
  SELECT constraint_name INTO unused
    FROM user_constraints
   WHERE table_name = 'FILESYSTEM'
     AND constraint_name = 'NN_FILESYSTEM_MOUNTPOINT';
EXCEPTION WHEN NO_DATA_FOUND THEN
  EXECUTE IMMEDIATE
    'ALTER TABLE FileSystem MODIFY
      (mountPoint CONSTRAINT NN_FileSystem_MountPoint NOT NULL)';
  EXECUTE IMMEDIATE
    'ALTER TABLE FileSystem ADD CONSTRAINT UN_FileSystem_DSMountPoint
       UNIQUE (diskServer, mountPoint)';
END;
/

/* #63103: Unable to clean Get/Update subRequest : "ORA-01403: no data found"
 *         caused by premature job termination (REMOVAL OF WORKAROUND)
 */
DECLARE
  unused VARCHAR2(30);
BEGIN
  SELECT trigger_name INTO unused
    FROM user_triggers
   WHERE trigger_name = 'TR_SUBREQUEST_63103';
  EXECUTE IMMEDIATE 'DROP TRIGGER TR_SUBREQUEST_63103';
EXCEPTION WHEN NO_DATA_FOUND THEN
  NULL;  -- Nothing
END;
/

/* #62314: draindiskserver restart option */
ALTER TABLE DrainingFileSystem DROP CONSTRAINT CK_DrainingFs_Status;
ALTER TABLE DrainingFileSystem
  ADD CONSTRAINT CK_DrainingFs_Status
  CHECK (status IN (0, 1, 2, 3, 4, 5, 6, 7));

/* Remove the tr_DrainingFileSystem_Delete trigger */
DROP TRIGGER tr_DrainingFileSystem_Delete;

/* Drop PutDoneStart table */
DROP TABLE PutDoneStart;

/* Update entries in Type2Obj */
DELETE FROM Type2Obj WHERE type = 93;
DELETE FROM Type2Obj WHERE type = 111;
INSERT INTO Type2Obj (type, object) VALUES (191, 'StageQueryResult');


/* Update and revalidation of PL-SQL code */
/******************************************/

/* SQL statement for the update trigger on the FileSystem table */
CREATE OR REPLACE TRIGGER tr_FileSystem_Update
BEFORE UPDATE OF status ON FileSystem
FOR EACH ROW WHEN (old.status != new.status)
BEGIN
  -- If the filesystem is coming back into PRODUCTION, initiate a consistency
  -- check for the diskcopies which reside on the filesystem.
  IF :old.status IN (1, 2) AND  -- DRAINING, DISABLED
     :new.status = 0 THEN       -- PRODUCTION
    checkFsBackInProd(:old.id);
  END IF;
  -- Cancel any ongoing draining operations if the filesystem is not in a
  -- DRAINING state
  IF :new.status != 1 THEN  -- DRAINING
    UPDATE DrainingFileSystem
       SET status = 3  -- INTERRUPTED
     WHERE fileSystem = :new.id
       AND status IN (0, 1, 2, 7);  -- CREATED, INITIALIZING, RUNNING, RESTART
  END IF;
END;
/

/* SQL statement for the update trigger on the DiskServer table */
CREATE OR REPLACE TRIGGER tr_DiskServer_Update
BEFORE UPDATE OF status ON DiskServer
FOR EACH ROW WHEN (old.status != new.status)
BEGIN
  -- If the diskserver is coming back into PRODUCTION, initiate a consistency
  -- check for all the diskcopies on its associated filesystems which are in
  -- a PRODUCTION.
  IF :old.status IN (1, 2) AND  -- DRAINING, DISABLED
     :new.status = 0 THEN       -- PRODUCTION
    FOR fs IN (SELECT id FROM FileSystem
                WHERE diskServer = :old.id
                  AND status = 0)  -- PRODUCTION
    LOOP
      checkFsBackInProd(fs.id);
    END LOOP;
  END IF;
  -- Cancel all draining operations if the diskserver is disabled.
  IF :new.status = 2 THEN  -- DISABLED
    UPDATE DrainingFileSystem
       SET status = 3  -- INTERRUPTED
     WHERE fileSystem IN
       (SELECT FileSystem.id FROM FileSystem
         WHERE FileSystem.diskServer = :new.id)
       AND status IN (0, 1, 2, 7);  -- CREATED, INITIALIZING, RUNNING, RESTART
  END IF;
  -- If the diskserver is in PRODUCTION cancel the draining operation of 
  -- filesystems not in DRAINING.
  IF :new.status = 0 THEN  -- PRODUCTION
    UPDATE DrainingFileSystem
       SET status = 3  -- INTERRUPTED
     WHERE fileSystem IN
       (SELECT FileSystem.id FROM FileSystem
         WHERE FileSystem.diskServer = :new.ID
           AND FileSystem.status != 1)  -- DRAINING
       AND status IN (0, 1, 2, 7);  -- CREATED, INITIALIZING, RUNNING, RESTART
  END IF; 
END;
/

/* PL/SQL method implementing stageRm */
CREATE OR REPLACE PROCEDURE stageRm (srId IN INTEGER,
                                     fid IN INTEGER,
                                     nh IN VARCHAR2,
                                     svcClassId IN INTEGER,
                                     ret OUT INTEGER) AS
  cfId INTEGER;
  scId INTEGER;
  nbRes INTEGER;
  dcsToRm "numList";
  srIds "numList";
  srType INTEGER;
  dcStatus INTEGER;
  nsHostName VARCHAR2(2048);
BEGIN
  ret := 0;
  -- Get the stager/nsHost configuration option
  nsHostName := getConfigOption('stager', 'nsHost', nh);
  BEGIN
    -- Lock the access to the CastorFile
    -- This, together with triggers will avoid new TapeCopies
    -- or DiskCopies to be added
    SELECT id INTO cfId FROM CastorFile
     WHERE fileId = fid AND nsHost = nsHostName FOR UPDATE;
  EXCEPTION WHEN NO_DATA_FOUND THEN
    -- This file does not exist in the stager catalog
    -- so we just fail the request
    UPDATE SubRequest
       SET status = 7,  -- FAILED
           errorCode = 2,  -- ENOENT
           errorMessage = 'File not found on disk cache'
     WHERE id = srId;
    RETURN;
  END;
  -- First select involved diskCopies
  scId := svcClassId;
  dcStatus := 0;
  IF scId > 0 THEN
    SELECT id BULK COLLECT INTO dcsToRm FROM (
      -- first physical diskcopies
      SELECT DC.id
        FROM DiskCopy DC, FileSystem, DiskPool2SvcClass DP2SC
       WHERE DC.castorFile = cfId
         AND DC.status IN (0, 6, 10)  -- STAGED, STAGEOUT, CANBEMIGR
         AND DC.fileSystem = FileSystem.id
         AND FileSystem.diskPool = DP2SC.parent
         AND DP2SC.child = scId)
    UNION ALL (
      -- and then diskcopies resulting from ongoing requests, for which the previous
      -- query wouldn't return any entry because of e.g. missing filesystem
      SELECT DC.id
        FROM (SELECT id FROM StagePrepareToPutRequest WHERE svcClass = scId UNION ALL
              SELECT id FROM StagePrepareToGetRequest WHERE svcClass = scId UNION ALL
              SELECT id FROM StagePrepareToUpdateRequest WHERE svcClass = scId UNION ALL
              SELECT id FROM StageRepackRequest WHERE svcClass = scId UNION ALL
              SELECT id FROM StagePutRequest WHERE svcClass = scId UNION ALL
              SELECT id FROM StageGetRequest WHERE svcClass = scId UNION ALL
              SELECT id FROM StageUpdateRequest WHERE svcClass = scId UNION ALL
              SELECT id FROM StageDiskCopyReplicaRequest WHERE svcClass = scId) Request,
             SubRequest, DiskCopy DC
       WHERE SubRequest.diskCopy = DC.id
         AND Request.id = SubRequest.request
         AND DC.castorFile = cfId
         AND DC.status IN (1, 2, 5, 11)  -- WAITDISK2DISKCOPY, WAITTAPERECALL, WAITFS, WAITFS_SCHEDULING
      );
    IF dcsToRm.COUNT = 0 THEN
      -- We didn't find anything on this svcClass, fail and return
      UPDATE SubRequest
         SET status = 7,  -- FAILED
             errorCode = 2,  -- ENOENT
             errorMessage = 'File not found on this service class'
       WHERE id = srId;
      RETURN;
    END IF;
    -- Check whether something else is left: if not, do as
    -- we are performing a stageRm everywhere.
    -- First select current status of the diskCopies: if CANBEMIGR,
    -- make sure we don't drop the last remaining valid migratable copy,
    -- i.e. exclude the disk only copies from the count.
    SELECT status INTO dcStatus
      FROM DiskCopy
     WHERE id = dcsToRm(1);
    IF dcStatus = 10 THEN  -- CANBEMIGR
      SELECT count(*) INTO nbRes FROM DiskCopy
       WHERE castorFile = cfId
         AND status = 10  -- CANBEMIGR
         AND id NOT IN (
           (SELECT /*+ CARDINALITY(dcidTable 5) */ *
              FROM TABLE(dcsToRm) dcidTable)
           UNION
           (SELECT DC.id     -- all diskcopies in Tape0 pools
              FROM DiskCopy DC, FileSystem, DiskPool2SvcClass D2S, SvcClass, FileClass
             WHERE DC.castorFile = cfId                         
               AND DC.fileSystem = FileSystem.id
               AND FileSystem.diskPool = D2S.parent
               AND D2S.child = SvcClass.id
               AND SvcClass.forcedFileClass = FileClass.id
               AND FileClass.nbCopies = 0));
    ELSE
      SELECT count(*) INTO nbRes FROM DiskCopy
         WHERE castorFile = cfId
           AND status IN (0, 2, 5, 6, 10, 11)  -- STAGED, WAITTAPERECALL, STAGEOUT, CANBEMIGR, WAITFS, WAITFS_SCHEDULING
           AND id NOT IN (SELECT /*+ CARDINALITY(dcidTable 5) */ *
                            FROM TABLE(dcsToRm) dcidTable);
    END IF;
    IF nbRes = 0 THEN
      -- nothing found, so we're dropping the last copy; then
      -- we need to perform all the checks to make sure we can
      -- allow the removal.
      scId := 0;
    END IF;
  END IF;

  IF scId = 0 THEN
    -- full cleanup is to be performed, do all necessary checks beforehand
    DECLARE
      segId INTEGER;
      unusedIds "numList";
    BEGIN
      -- check if removal is possible for migration
      SELECT count(*) INTO nbRes FROM DiskCopy
       WHERE status = 10 -- DISKCOPY_CANBEMIGR
         AND castorFile = cfId;
      IF nbRes > 0 THEN
        -- We found something, thus we cannot remove
        UPDATE SubRequest
           SET status = 7,  -- FAILED
               errorCode = 16,  -- EBUSY
               errorMessage = 'The file is not yet migrated'
         WHERE id = srId;
        RETURN;
      END IF;
      -- Stop ongoing recalls if stageRm either everywhere or the only available diskcopy.
      -- This is not entirely clean: a proper operation here should be to
      -- drop the SubRequest waiting for recall but keep the recall if somebody
      -- else is doing it, and taking care of other WAITSUBREQ requests as well...
      -- but it's fair enough, provided that the last stageRm will cleanup everything.
      -- XXX First lock all segments for the file. Note that
      -- XXX this step should be dropped once the tapeGateway
      -- XXX is deployed. The current recaller does not take
      -- XXX the proper lock on the castorFiles, hence we
      -- XXX need this here
      SELECT Segment.id BULK COLLECT INTO unusedIds
        FROM Segment, TapeCopy
       WHERE TapeCopy.castorfile = cfId
         AND TapeCopy.id = Segment.copy
       ORDER BY Segment.id
      FOR UPDATE OF Segment.id;
      -- Check whether we have any segment in SELECTED
      SELECT segment.id INTO segId
        FROM Segment, TapeCopy
       WHERE TapeCopy.castorfile = cfId
         AND TapeCopy.id = Segment.copy
         AND Segment.status = 7 -- SELECTED
         AND ROWNUM < 2;
      -- Something is running, so give up
      UPDATE SubRequest
         SET status = 7,  -- FAILED
             errorCode = 16,  -- EBUSY
             errorMessage = 'The file is being recalled from tape'
       WHERE id = srId;
      RETURN;
    EXCEPTION WHEN NO_DATA_FOUND THEN
      -- Nothing running. We still may have found nothing at all...
      SELECT count(*) INTO nbRes FROM DiskCopy
       WHERE castorFile = cfId
         AND status NOT IN (4, 7, 9);  -- anything but FAILED, INVALID, BEINGDELETED
      IF nbRes = 0 THEN
        UPDATE SubRequest
           SET status = 7,  -- FAILED
               errorCode = 2,  -- ENOENT
               errorMessage = 'File not found on disk cache'
         WHERE id = srId;
        RETURN;
      END IF;
      
      deleteTapeCopies(cfId);
      -- Invalidate the DiskCopies
      UPDATE DiskCopy
         SET status = 7  -- INVALID
       WHERE status = 2  -- WAITTAPERECALL
         AND castorFile = cfId;
      -- Mark the 'recall' SubRequests as failed
      -- so that clients eventually get an answer
      UPDATE SubRequest
         SET status = 7,  -- FAILED
             errorCode = 4,  -- EINTR
             errorMessage = 'Recall canceled by another user request'
       WHERE castorFile = cfId and status IN (4, 5);   -- WAITTAPERECALL, WAITSUBREQ
      -- Reselect what needs to be removed
      SELECT id BULK COLLECT INTO dcsToRm
        FROM DiskCopy
       WHERE castorFile = cfId
         AND status IN (0, 1, 5, 6, 10, 11);  -- STAGED, WAITDISK2DISKCOPY, WAITFS, STAGEOUT, CANBEMIGR, WAITFS_SCHEDULING
    END;
  END IF;

  -- Now perform the remove:
  -- mark all get/put requests for those diskcopies
  -- and the ones waiting on them as failed
  -- so that clients eventually get an answer
  SELECT /*+ INDEX(SR I_SubRequest_DiskCopy) */ id
    BULK COLLECT INTO srIds
    FROM SubRequest SR
   WHERE diskcopy IN
     (SELECT /*+ CARDINALITY(dcidTable 5) */ * 	 
        FROM TABLE(dcsToRm) dcidTable) 	 
     AND status IN (0, 1, 2, 5, 6, 13); -- START, WAITSUBREQ, READY, READYFORSCHED
  IF srIds.COUNT > 0 THEN
    FOR i IN srIds.FIRST .. srIds.LAST LOOP
      SELECT type INTO srType
        FROM SubRequest, Id2Type
       WHERE SubRequest.request = Id2Type.id
         AND SubRequest.id = srIds(i);
      UPDATE SubRequest
         SET status = CASE
             WHEN status IN (6, 13) AND srType = 133 THEN 9 ELSE 7
             END,  -- FAILED_FINISHED for DiskCopyReplicaRequests in status READYFORSCHED or READY, otherwise FAILED
             -- this so that user requests in status WAITSUBREQ are always marked FAILED even if they wait on a replication
             errorCode = 4,  -- EINTR
             errorMessage = 'Canceled by another user request'
       WHERE id = srIds(i) OR parent = srIds(i);
    END LOOP;
  END IF;
  -- Set selected DiskCopies to either INVALID or FAILED;
  -- WAITDISK2DISKCOPY's have a dedicated handling (see bug #63348)
  FOR i IN dcsToRm.FIRST .. dcsToRm.LAST LOOP
    SELECT status INTO dcStatus
      FROM DiskCopy
     WHERE id = dcsToRm(i);
    IF dcStatus = 1 THEN  -- WAITDISK2DISKCOPY
      disk2DiskCopyFailed(dcsToRm(i), 0);
    ELSE
      UPDATE DiskCopy
         -- WAITTAPERECALL,WAITFS[_SCHED] -> FAILED, others -> INVALID
         SET status = decode(status, 2,4, 5,4, 11,4, 7)
       WHERE id = dcsToRm(i);
    END IF;
  END LOOP;
  ret := 1;  -- ok
END;
/

/* PL/SQL method implementing getUpdateStart */
CREATE OR REPLACE PROCEDURE getUpdateStart
        (srId IN INTEGER, selectedDiskServer IN VARCHAR2, selectedMountPoint IN VARCHAR2,
         dci OUT INTEGER, rpath OUT VARCHAR2, rstatus OUT NUMBER, reuid OUT INTEGER,
         regid OUT INTEGER, diskCopySize OUT NUMBER) AS
  cfid INTEGER;
  fid INTEGER;
  dcId INTEGER;
  fsId INTEGER;
  dcIdInReq INTEGER;
  nh VARCHAR2(2048);
  fileSize INTEGER;
  srSvcClass INTEGER;
  proto VARCHAR2(2048);
  isUpd NUMBER;
  nbAc NUMBER;
  gcw NUMBER;
  gcwProc VARCHAR2(2048);
  cTime NUMBER;
BEGIN
  -- Get data
  SELECT euid, egid, svcClass, upd, diskCopy
    INTO reuid, regid, srSvcClass, isUpd, dcIdInReq
    FROM SubRequest,
        (SELECT id, euid, egid, svcClass, 0 AS upd FROM StageGetRequest UNION ALL
         SELECT id, euid, egid, svcClass, 1 AS upd FROM StageUpdateRequest) Request
   WHERE SubRequest.request = Request.id AND SubRequest.id = srId;
  -- Take a lock on the CastorFile. Associated with triggers,
  -- this guarantees we are the only ones dealing with its copies
  SELECT CastorFile.fileSize, CastorFile.id
    INTO fileSize, cfId
    FROM CastorFile, SubRequest
   WHERE CastorFile.id = SubRequest.castorFile
     AND SubRequest.id = srId FOR UPDATE;
  -- Check to see if the proposed diskserver and filesystem selected by the
  -- scheduler to run the job is in the correct service class.
  BEGIN
    SELECT FileSystem.id INTO fsId
      FROM DiskServer, FileSystem, DiskPool2SvcClass
     WHERE FileSystem.diskserver = DiskServer.id
       AND FileSystem.diskPool = DiskPool2SvcClass.parent
       AND DiskPool2SvcClass.child = srSvcClass
       AND DiskServer.name = selectedDiskServer
       AND FileSystem.mountPoint = selectedMountPoint;
  EXCEPTION WHEN NO_DATA_FOUND THEN
    raise_application_error(-20114, 'Job scheduled to the wrong service class. Giving up.');
  END;
  -- Try to find local DiskCopy
  SELECT /*+ INDEX(DiskCopy I_DiskCopy_Castorfile) */ id, nbCopyAccesses, gcWeight, creationTime
    INTO dcId, nbac, gcw, cTime
    FROM DiskCopy
   WHERE DiskCopy.castorfile = cfId
     AND DiskCopy.filesystem = fsId
     AND DiskCopy.status IN (0, 6, 10) -- STAGED, STAGEOUT, CANBEMIGR
     AND ROWNUM < 2;
  -- We found it, so we are settled and we'll use the local copy.
  -- It might happen that we have more than one, because LSF may have
  -- scheduled a replication on a fileSystem which already had a previous diskcopy.
  -- We don't care and we randomly took the first one.
  -- First we will compute the new gcWeight of the diskcopy
  IF nbac = 0 THEN
    gcwProc := castorGC.getFirstAccessHook(srSvcClass);
    IF gcwProc IS NOT NULL THEN
      EXECUTE IMMEDIATE 'BEGIN :newGcw := ' || gcwProc || '(:oldGcw, :cTime); END;'
        USING OUT gcw, IN gcw, IN cTime;
    END IF;
  ELSE
    gcwProc := castorGC.getAccessHook(srSvcClass);
    IF gcwProc IS NOT NULL THEN
      EXECUTE IMMEDIATE 'BEGIN :newGcw := ' || gcwProc || '(:oldGcw, :cTime, :nbAc); END;'
        USING OUT gcw, IN gcw, IN cTime, IN nbac;
    END IF;
  END IF;
  -- Here we also update the gcWeight taking into account the new lastAccessTime
  -- and the weightForAccess from our svcClass: this is added as a bonus to
  -- the selected diskCopy.
  UPDATE DiskCopy
     SET gcWeight = gcw,
         lastAccessTime = getTime(),
         nbCopyAccesses = nbCopyAccesses + 1
   WHERE id = dcId
  RETURNING id, path, status, diskCopySize INTO dci, rpath, rstatus, diskCopySize;
  -- Let's update the SubRequest and set the link with the DiskCopy
  UPDATE SubRequest SET DiskCopy = dci WHERE id = srId RETURNING protocol INTO proto;
  -- In case of an update, if the protocol used does not support
  -- updates properly (via firstByteWritten call), we should
  -- call firstByteWritten now and consider that the file is being
  -- modified.
  IF isUpd = 1 THEN
    handleProtoNoUpd(srId, proto);
  END IF;
EXCEPTION WHEN NO_DATA_FOUND THEN
  -- No disk copy found on selected FileSystem. This can happen in 2 cases :
  --  + either a diskcopy was available and got disabled before this job
  --    was scheduled. Bad luck, we fail the request, the user will have to retry
  --  + or we are an update creating a file and there is a diskcopy in WAITFS
  --    or WAITFS_SCHEDULING associated to us. Then we have to call putStart
  -- So we first check the update hypothesis
  IF isUpd = 1 AND dcIdInReq IS NOT NULL THEN
    DECLARE
      stat NUMBER;
    BEGIN
      SELECT status INTO stat FROM DiskCopy WHERE id = dcIdInReq;
      IF stat IN (5, 11) THEN -- WAITFS, WAITFS_SCHEDULING
        -- it is an update creating a file, let's call putStart
        putStart(srId, selectedDiskServer, selectedMountPoint, dci, rstatus, rpath);
        RETURN;
      END IF;
    END;
  END IF;
  -- It was not an update creating a file, so we fail
  raise_application_error(-20114, 'File invalidated while queuing in the scheduler, please try again');
END;
/

/* PL/SQL method implementing getUpdateStart */
CREATE OR REPLACE PROCEDURE getUpdateStart
        (srId IN INTEGER, selectedDiskServer IN VARCHAR2, selectedMountPoint IN VARCHAR2,
         dci OUT INTEGER, rpath OUT VARCHAR2, rstatus OUT NUMBER, reuid OUT INTEGER,
         regid OUT INTEGER, diskCopySize OUT NUMBER) AS
  cfid INTEGER;
  fid INTEGER;
  dcId INTEGER;
  fsId INTEGER;
  dcIdInReq INTEGER;
  nh VARCHAR2(2048);
  fileSize INTEGER;
  srSvcClass INTEGER;
  proto VARCHAR2(2048);
  isUpd NUMBER;
  nbAc NUMBER;
  gcw NUMBER;
  gcwProc VARCHAR2(2048);
  cTime NUMBER;
BEGIN
  -- Get data
  SELECT euid, egid, svcClass, upd, diskCopy
    INTO reuid, regid, srSvcClass, isUpd, dcIdInReq
    FROM SubRequest,
        (SELECT id, euid, egid, svcClass, 0 AS upd FROM StageGetRequest UNION ALL
         SELECT id, euid, egid, svcClass, 1 AS upd FROM StageUpdateRequest) Request
   WHERE SubRequest.request = Request.id AND SubRequest.id = srId;
  -- Take a lock on the CastorFile. Associated with triggers,
  -- this guarantees we are the only ones dealing with its copies
  SELECT CastorFile.fileSize, CastorFile.id
    INTO fileSize, cfId
    FROM CastorFile, SubRequest
   WHERE CastorFile.id = SubRequest.castorFile
     AND SubRequest.id = srId FOR UPDATE;
  -- Check to see if the proposed diskserver and filesystem selected by the
  -- scheduler to run the job is in the correct service class.
  BEGIN
    SELECT FileSystem.id INTO fsId
      FROM DiskServer, FileSystem, DiskPool2SvcClass
     WHERE FileSystem.diskserver = DiskServer.id
       AND FileSystem.diskPool = DiskPool2SvcClass.parent
       AND DiskPool2SvcClass.child = srSvcClass
       AND DiskServer.name = selectedDiskServer
       AND FileSystem.mountPoint = selectedMountPoint;
  EXCEPTION WHEN NO_DATA_FOUND THEN
    raise_application_error(-20114, 'Job scheduled to the wrong service class. Giving up.');
  END;
  -- Try to find local DiskCopy
  SELECT /*+ INDEX(DiskCopy I_DiskCopy_Castorfile) */ id, nbCopyAccesses, gcWeight, creationTime
    INTO dcId, nbac, gcw, cTime
    FROM DiskCopy
   WHERE DiskCopy.castorfile = cfId
     AND DiskCopy.filesystem = fsId
     AND DiskCopy.status IN (0, 6, 10) -- STAGED, STAGEOUT, CANBEMIGR
     AND ROWNUM < 2;
  -- We found it, so we are settled and we'll use the local copy.
  -- It might happen that we have more than one, because LSF may have
  -- scheduled a replication on a fileSystem which already had a previous diskcopy.
  -- We don't care and we randomly took the first one.
  -- First we will compute the new gcWeight of the diskcopy
  IF nbac = 0 THEN
    gcwProc := castorGC.getFirstAccessHook(srSvcClass);
    IF gcwProc IS NOT NULL THEN
      EXECUTE IMMEDIATE 'BEGIN :newGcw := ' || gcwProc || '(:oldGcw, :cTime); END;'
        USING OUT gcw, IN gcw, IN cTime;
    END IF;
  ELSE
    gcwProc := castorGC.getAccessHook(srSvcClass);
    IF gcwProc IS NOT NULL THEN
      EXECUTE IMMEDIATE 'BEGIN :newGcw := ' || gcwProc || '(:oldGcw, :cTime, :nbAc); END;'
        USING OUT gcw, IN gcw, IN cTime, IN nbac;
    END IF;
  END IF;
  -- Here we also update the gcWeight taking into account the new lastAccessTime
  -- and the weightForAccess from our svcClass: this is added as a bonus to
  -- the selected diskCopy.
  UPDATE DiskCopy
     SET gcWeight = gcw,
         lastAccessTime = getTime(),
         nbCopyAccesses = nbCopyAccesses + 1
   WHERE id = dcId
  RETURNING id, path, status, diskCopySize INTO dci, rpath, rstatus, diskCopySize;
  -- Let's update the SubRequest and set the link with the DiskCopy
  UPDATE SubRequest SET DiskCopy = dci WHERE id = srId RETURNING protocol INTO proto;
  -- In case of an update, if the protocol used does not support
  -- updates properly (via firstByteWritten call), we should
  -- call firstByteWritten now and consider that the file is being
  -- modified.
  IF isUpd = 1 THEN
    handleProtoNoUpd(srId, proto);
  END IF;
EXCEPTION WHEN NO_DATA_FOUND THEN
  -- No disk copy found on selected FileSystem. This can happen in 2 cases :
  --  + either a diskcopy was available and got disabled before this job
  --    was scheduled. Bad luck, we fail the request, the user will have to retry
  --  + or we are an update creating a file and there is a diskcopy in WAITFS
  --    or WAITFS_SCHEDULING associated to us. Then we have to call putStart
  -- So we first check the update hypothesis
  IF isUpd = 1 AND dcIdInReq IS NOT NULL THEN
    DECLARE
      stat NUMBER;
    BEGIN
      SELECT status INTO stat FROM DiskCopy WHERE id = dcIdInReq;
      IF stat IN (5, 11) THEN -- WAITFS, WAITFS_SCHEDULING
        -- it is an update creating a file, let's call putStart
        putStart(srId, selectedDiskServer, selectedMountPoint, dci, rstatus, rpath);
        RETURN;
      END IF;
    END;
  END IF;
  -- It was not an update creating a file, so we fail
  raise_application_error(-20114, 'File invalidated while queuing in the scheduler, please try again');
END;
/

/* PL/SQL method implementing disk2DiskCopyFailed */
CREATE OR REPLACE PROCEDURE disk2DiskCopyFailed
(dcId IN INTEGER, enoent IN INTEGER) AS
  fsId    NUMBER;
  cfId    NUMBER;
  ouid    INTEGER;
  ogid    INTEGER;
  srcDcId NUMBER;
  srcFsId NUMBER;
  srId    NUMBER;
BEGIN
  fsId := 0;
  srcFsId := -1;
  -- Lock the CastorFile
  SELECT id INTO cfId FROM CastorFile
   WHERE id = 
    (SELECT castorFile 
       FROM DiskCopy
      WHERE id = dcId) FOR UPDATE;
  IF enoent = 1 THEN
    -- Set all diskcopies to FAILED. We're preemptying the NS synchronization here
    UPDATE DiskCopy SET status = 4 -- FAILED
     WHERE castorFile =
       (SELECT castorFile FROM DiskCopy WHERE id = dcId); 
  ELSE
    -- Set the diskcopy status to INVALID so that it will be garbage collected
    -- at a later date.
    UPDATE DiskCopy SET status = 7 -- INVALID
     WHERE status = 1 -- WAITDISK2DISKCOPY
       AND id = dcId
     RETURNING fileSystem, castorFile, owneruid, ownergid
      INTO fsId, cfId, ouid, ogid;
  END IF;
  BEGIN
    -- Determine the source diskcopy and filesystem involved in the replication
    SELECT sourceDiskCopy, fileSystem
      INTO srcDcId, srcFsId
      FROM DiskCopy, StageDiskCopyReplicaRequest
     WHERE StageDiskCopyReplicaRequest.sourceDiskCopy = DiskCopy.id
       AND StageDiskCopyReplicaRequest.destDiskCopy = dcId;
    -- Restart all entries in the snapshot of files to drain that may be
    -- waiting on the replication of the source diskcopy.
    UPDATE DrainingDiskCopy
       SET status = 1,  -- RESTARTED
           parent = 0
     WHERE status = 3  -- RUNNING
       AND (diskCopy = srcDcId
        OR  parent = srcDcId);
    -- Get the corresponding subRequest, if it exists
    SELECT id INTO srId
      FROM SubRequest
     WHERE diskCopy = dcId
       AND status IN (6, 14); -- READY, BEINGSHCED
    -- Wake up other subrequests waiting on it
    UPDATE SubRequest SET status = 1, -- RESTART
                          parent = 0
     WHERE parent = srId;
    -- Fail it
    archiveSubReq(srId, 9); -- FAILED_FINISHED
    -- If no filesystem was set on the diskcopy then we can safely delete it
    -- without waiting for garbage collection as the transfer was never started
    IF fsId = 0 THEN
      DELETE FROM DiskCopy WHERE id = dcId;
      DELETE FROM Id2Type WHERE id = dcId;
    END IF;
    -- Trigger the creation of additional copies of the file, if necessary.
    -- Note: We do this also on failure to be able to recover from transient
    -- errors, e.g. timeouts while waiting to be scheduled, but we don't on ENOENT.
    IF enoent = 0 THEN
      replicateOnClose(cfId, ouid, ogid);
    END IF;
  EXCEPTION WHEN NO_DATA_FOUND THEN
    -- SubRequest not found, don't trigger replicateOnClose here
    -- as we may have been restarted
    NULL;
  END;
  -- Continue draining process
  drainFileSystem(srcFsId);
  -- WARNING: previous call to drainFileSystem has a COMMIT inside. So all
  -- locks have been released!!
END;
/

/* PL/SQL method implementing putFailedProc */
CREATE OR REPLACE PROCEDURE putFailedProc(srId IN NUMBER) AS
  dcId INTEGER;
  cfId INTEGER;
  unused INTEGER;
BEGIN
  SELECT diskCopy, castorFile INTO dcId, cfId
    FROM SubRequest
   WHERE id = srId;
  -- Fail the subRequest
  UPDATE SubRequest
     SET status = 7 -- FAILED
   WHERE id = srId;
  -- Determine the context (Put inside PrepareToPut/Update ?)
  BEGIN
    -- Check that there is a PrepareToPut/Update going on. There can be only a single one
    -- or none. If there was a PrepareTo, any subsequent PPut would be rejected and any
    -- subsequent PUpdate would be directly archived (cf. processPrepareRequest).
    SELECT SubRequest.id INTO unused
      FROM (SELECT id FROM StagePrepareToPutRequest UNION ALL
            SELECT id FROM StagePrepareToUpdateRequest) PrepareRequest, SubRequest
     WHERE SubRequest.castorFile = cfId
       AND PrepareRequest.id = SubRequest.request
       AND SubRequest.status = 6; -- READY
    -- The select worked out, so we have a prepareToPut/Update
    -- In such a case, we do not cleanup DiskCopy and CastorFile
    -- but we have to wake up a potential waiting putDone
    UPDATE SubRequest SET status = 1, parent = 0 -- RESTART
     WHERE id IN
      (SELECT SubRequest.id
         FROM StagePutDoneRequest, SubRequest
        WHERE SubRequest.CastorFile = cfId
          AND StagePutDoneRequest.id = SubRequest.request
          AND SubRequest.status = 5); -- WAITSUBREQ
  EXCEPTION WHEN NO_DATA_FOUND THEN
    -- This means we are a standalone put
    -- thus cleanup DiskCopy and maybe the CastorFile
    -- (the physical file is dropped by the job)
    DELETE FROM DiskCopy WHERE id = dcId;
    DELETE FROM Id2Type WHERE id = dcId;
    deleteCastorFile(cfId);
  END;
END;
/

/* PL/SQL function implementing checkAvailOfSchedulerRFS, this function can be
 * used to determine if any of the requested filesystems specified by a
 * transfer are available. Returns 0 if at least one requested filesystem is
 * available otherwise 1.
 */
CREATE OR REPLACE FUNCTION checkAvailOfSchedulerRFS
  (rfs IN VARCHAR2, reqType IN NUMBER)
RETURN NUMBER IS
  rtn NUMBER;
BEGIN
  -- Count the number of requested filesystems which are available
  SELECT count(*) INTO rtn
    FROM DiskServer, FileSystem 
   WHERE DiskServer.id = FileSystem.diskServer
     AND DiskServer.name || ':' || FileSystem.mountPoint IN
       (SELECT /*+ CARDINALITY(rfsTable 10) */ * 
          FROM TABLE (strTokenizer(rfs, '|')) rfsTable)
     -- For a requested filesystem to be available the following criteria
     -- must be meet:
     --  - The diskserver and filesystem must not be in a DISABLED state
     --  - For StageDiskCopyReplicaRequests all other states are accepted
     --  - For all other requests the diskserver and filesystem must be in
     --    PRODUCTION
     AND decode(DiskServer.status, 2, 1,    -- Exclude DISABLED Diskservers
           decode(FileSystem.status, 2, 1,  -- Exclude DISABLED Filesystems
             decode(reqType, 133, 0,
                 decode(FileSystem.status + DiskServer.status, 0, 0, 1)))) = 0;
  IF rtn > 0 THEN
    RETURN 0;  -- We found some available requested filesystems
  END IF;
  RETURN 1;
END;
/

/* PL/SQL method implementing getSchedulerJobs. This method lists all known
 * transfers and whether they should be terminated because no space exists
 * in the target service class or none of the requested filesystems are
 * available.
 */
CREATE OR REPLACE PROCEDURE getSchedulerJobs
  (transfers OUT castor.SchedulerJobs_Cur) AS
BEGIN
  OPEN transfers FOR
    -- Use the NO_MERGE hint to prevent Oracle from executing the
    -- checkFailJobsWhenNoSpace function for every row in the output. In
    -- situations where there are many PENDING transfers in the scheduler
    -- this can be extremely inefficient and expensive.
    SELECT /*+ NO_MERGE(NSSvc) */
           SR.subReqId, Request.reqId, NSSvc.NoSpace,
           -- If there are no requested filesystems, refer to the NFSSvc
           -- output otherwise call the checkAvailOfSchedulerRFS function
           decode(SR.requestedFileSystems, NULL, NFSSvc.NoFSAvail,
             checkAvailOfSchedulerRFS(SR.requestedFileSystems,
                                      Request.reqType)) NoFSAvail
      FROM SubRequest SR,
        -- Union of all requests that could result in scheduler transfers
        (SELECT id, svcClass, reqid, 40  AS reqType
           FROM StagePutRequest                     UNION ALL
         SELECT id, svcClass, reqid, 133 AS reqType
           FROM StageDiskCopyReplicaRequest         UNION ALL
         SELECT id, svcClass, reqid, 35  AS reqType
           FROM StageGetRequest                     UNION ALL
         SELECT id, svcClass, reqid, 44  AS reqType
           FROM StageUpdateRequest) Request,
        -- Table of all service classes with a boolean flag to indicate
        -- if space is available
        (SELECT id, checkFailJobsWhenNoSpace(id) NoSpace
           FROM SvcClass) NSSvc,
        -- Table of all service classes with a boolean flag to indicate
        -- if there are any filesystems in PRODUCTION
        (SELECT id, nvl(NoFSAvail, 1) NoFSAvail FROM SvcClass
           LEFT JOIN 
             (SELECT DP2Svc.CHILD, decode(count(*), 0, 1, 0) NoFSAvail
                FROM DiskServer DS, FileSystem FS, DiskPool2SvcClass DP2Svc
               WHERE DS.ID = FS.diskServer
                 AND DS.status = 0  -- DISKSERVER_PRODUCTION
                 AND FS.diskPool = DP2Svc.parent
                 AND FS.status = 0  -- FILESYSTEM_PRODUCTION
               GROUP BY DP2Svc.child) results
             ON SvcClass.id = results.child) NFSSvc
     WHERE SR.status = 6  -- READY
       AND SR.request = Request.id
       AND SR.lastModificationTime < getTime() - 60
       AND NSSvc.id = Request.svcClass
       AND NFSSvc.id = Request.svcClass;
END;
/

/* PL/SQL method implementing rtcpclientdCleanUp */
CREATE OR REPLACE PROCEDURE rtcpclientdCleanUp AS
  tpIds "numList";
  unused VARCHAR2(2048);
BEGIN
  SELECT value INTO unused
    FROM CastorConfig
   WHERE class = 'tape'
     AND KEY = 'interfaceDaemon'
     AND value = 'rtcpclientd';
  -- JUST rtcpclientd
  -- Deal with Migrations
  -- 1) Ressurect tapecopies for migration
  UPDATE TapeCopy SET status = 1 WHERE status = 3;
  -- 2) Clean up the streams
  UPDATE Stream SET status = 0 
   WHERE status NOT IN (0, 5, 6, 7) --PENDING, CREATED, STOPPED, WAITPOLICY
  RETURNING tape BULK COLLECT INTO tpIds;
  UPDATE Stream SET tape = NULL WHERE tape != 0;
  -- 3) Reset the tape for migration
  FORALL i IN tpIds.FIRST .. tpIds.LAST  
    UPDATE tape SET stream = 0, status = 0
     WHERE status IN (2, 3, 4) AND id = tpIds(i);

  -- Deal with Recalls
  UPDATE Segment SET status = 0
   WHERE status = 7; -- Resurrect SELECTED segment
  UPDATE Tape SET status = 1
   WHERE tpmode = 0 AND status IN (2, 3, 4); -- Resurrect the tapes running for recall
  UPDATE Tape A SET status = 8 
   WHERE status IN (0, 6, 7) AND EXISTS
    (SELECT id FROM Segment WHERE status = 0 AND tape = A.id);
  COMMIT;
END;
/

/* GenericInputForMigrationPolicy */
CREATE OR REPLACE PROCEDURE inputForMigrationPolicy
(svcclassName IN VARCHAR2,
 policyName OUT NOCOPY VARCHAR2,
 svcId OUT NUMBER,
 dbInfo OUT castorTape.DbMigrationInfo_Cur) AS
 unused VARCHAR2(2048);
BEGIN
  BEGIN
    SELECT value INTO unused
      FROM CastorConfig
     WHERE class = 'tape'
       AND key   = 'interfaceDaemon'
       AND value = 'tapegatewayd';
  EXCEPTION WHEN NO_DATA_FOUND THEN  -- rtcpclientd
    inputMigrPolicyRtcp(svcclassName, policyName, svcId, dbInfo);  
    RETURN;
  END;
  -- tapegateway
  inputMigrPolicyGateway(svcclassName, policyName, svcId, dbInfo);
END;
/

/* generic attach tapecopies to stream */
CREATE OR REPLACE PROCEDURE attachTapeCopiesToStreams 
(tapeCopyIds IN castor."cnumList",
 tapePoolId IN NUMBER) AS
  unused VARCHAR2(2048);
BEGIN
  BEGIN
    SELECT value INTO unused
      FROM CastorConfig
     WHERE class = 'tape'
       AND key   = 'interfaceDaemon'
       AND value = 'tapegatewayd';
  EXCEPTION WHEN NO_DATA_FOUND THEN  -- rtcpclientd
    attachTCRtcp(tapeCopyIds, tapePoolId);
    RETURN;
  END;
  -- tapegateway
  attachTCGateway(tapeCopyIds, tapePoolId);
END;
/

/*
restartStuckRecalls is a wrokaround procedure required by the rtcpclientd
daemon.

Restart the (recall) segments that are recognized as stuck.
This workaround (sr #112306: locking issue in CMS stager)
will be dropped as soon as the TapeGateway will be used in production.

Notes for query readability:
TAPE status:    (0)TAPE_UNUSED, (1)TAPE_PENDING, (2)TAPE_WAITDRIVE, 
                (3)TAPE_WAITMOUNT, (6)TAPE_FAILED
SEGMENT status: (0)SEGMENT_UNPROCESSED, (7)SEGMENT_SELECTED
*/
CREATE OR REPLACE PROCEDURE restartStuckRecalls AS
  unused VARCHAR2(2048);
BEGIN
  -- Do nothing and return if the tape gateway is running
  BEGIN
    SELECT value INTO unused
      FROM CastorConfig
     WHERE class = 'tape'
       AND key   = 'interfaceDaemon'
       AND value = 'tapegatewayd';
     RETURN;
  EXCEPTION WHEN NO_DATA_FOUND THEN  -- rtcpclientd
    -- Do nothing and continue
    NULL;
  END;

  -- Notes for query readability:
  -- TAPE status:    (0)TAPE_UNUSED, (1)TAPE_PENDING, (2)TAPE_WAITDRIVE, 
  --                 (3)TAPE_WAITMOUNT, (6)TAPE_FAILED
  -- SEGMENT status: (0)SEGMENT_UNPROCESSED, (7)SEGMENT_SELECTED

  -- Mark as unused all of the recall tapes whose state maybe stuck due to an
  -- rtcpclientd crash. Such tapes will be pending, waiting for a drive, or
  -- waiting for a mount, and will be associated with segments that are neither
  -- un-processed nor selected.
  UPDATE tape SET status=0 WHERE tpmode = 0 AND status IN (1,2,3)
    AND id NOT IN (
     SELECT tape FROM segment WHERE status IN (0,7));

  -- Mark as unprocessed all recall segments that are marked as being selected
  -- and are associated with unused or failed recall tapes that have 1 or more
  -- unprocessed or selected segments.
  UPDATE segment SET status = 0 WHERE status = 7 and tape IN (
    SELECT id FROM tape WHERE tpmode = 0 AND status IN (0, 6) AND id IN (
      SELECT tape FROM segment WHERE status IN (0, 7) ) );

  -- Mark as pending all recall tapes that are unused or failed, and have
  -- unprocessed and selected segments.
  UPDATE tape SET status = 1 WHERE tpmode = 0 AND status IN (0, 6) AND id IN (
    SELECT tape FROM segment WHERE status IN (0, 7));

  COMMIT;
END restartStuckRecalls;
/

/* SQL statement for the creation of the DrainingOverview view */
CREATE OR REPLACE VIEW DrainingOverview
AS
  SELECT DS.name DiskServer,
         FS.mountPoint MountPoint,
         -- Determine the status of the filesystem being drained. If it's in a
         -- PRODUCTION status then the status of the diskserver is taken.
         decode(
           decode(FS.status, 0, DS.status, FS.status),
                  0, 'PRODUCTION',
                  1, 'DRAINING',
                  2, 'DISABLED', 'UNKNOWN') FileSystemStatus,
         DFS.username Username,
         DFS.machine Machine,
         getTimeString(ceil(DFS.creationTime)) Created,
         DFS.maxTransfers MaxTransfers,
         DFS.totalFiles TotalFiles,
         sizeOfFmtSI(DFS.totalBytes) TotalSize,
         -- Translate the fileMask value to string
         decode(DFS.fileMask,
                0, 'STAGED',
                1, 'CANBEMIGR',
                2, 'ALL', 'UNKNOWN') FileMask,
         -- Translate the autoDelete value to a string
         decode(DFS.autoDelete, 0, 'NO', 1, 'YES', 'UNKNOWN') AutoDelete,
         -- The target service class
         SVC.name SvcClass,
         -- Determine the status of the draining process. If the last update
         -- time is more than 60 minutes ago the process is considered to be
         -- STALLED.
         decode(DFS.status, 0, 'CREATED', 1, 'INITIALIZING', 2,
           decode(sign((getTime() - 3600) - DFS.lastUpdateTime),
                  -1, 'RUNNING', 'STALLED'),
                   3, 'INTERRUPTED',
                   4, 'FAILED',
                   5, 'COMPLETED',
                   6, 'DELETING',
                   7, 'RESTART', 'UNKNOWN') Status,
         nvl(DDCS.filesRemaining, 0) FilesRemaining,
         sizeOfFmtSI(nvl(DDCS.bytesRemaining, 0)) SizeRemaining,
         nvl(DDCS.running, 0) Running,
         nvl(DDCS.failed, 0) Failed,
         -- Calculate how long the process for draining the filesystem has been
         -- running. If the process is in a CREATED or INITIALIZING status
         -- 00 00:00:00 will be returned.
         decode(DFS.status, 0, '00 00:00:00', 1, '00 00:00:00',
           decode(sign(DFS.status - 3), -1,
             getInterval(DFS.startTime, gettime()),
               getInterval(DFS.startTime, DFS.lastUpdateTime))) RunTime,
         -- Calculate how far the process has gotten as a percentage of the data
         -- already transferred. If the process is in a CREATED, INITIALIZING,
         -- DELETING or RESTART status N/A will be returned.
         decode(DFS.status, 0, 'N/A', 1, 'N/A', 6, 'N/A', 7, 'N/A',
           decode(DFS.totalBytes, 0, '100%',
             concat(to_char(
               floor(((DFS.totalBytes - nvl(DDCS.bytesRemaining, 0)) /
                       DFS.totalBytes) * 100)), '%'))) Progress,
         -- Calculate the estimated time to completion in seconds if the process
         -- is in a RUNNING status and more than 10% of the data has already by
         -- transferred.
         decode(DFS.status, 2, decode(DFS.totalBytes, 0, 'N/A',
           decode(sign((getTime() - 3600) - DFS.lastUpdateTime), -1,
             decode(sign((((DFS.totalBytes - nvl(DDCS.bytesRemaining, 0)) /
                            DFS.totalBytes) * 100) - 10), -1, 'N/A',
               getInterval(0, trunc(DDCS.bytesRemaining / ((DFS.totalBytes -
                           nvl(DDCS.bytesRemaining, 0)) /
                           (getTime() - DFS.startTime))))), 'N/A')), 'N/A') ETC
    FROM (
      SELECT fileSystem,
             max(decode(status, 3, nbFiles, 0)) Running,
             max(decode(status, 4, nbFiles, 0)) Failed,
             sum(nbFiles) FilesRemaining,
             sum(totalFileSize) BytesRemaining
        FROM (
          SELECT fileSystem, status, count(*) nbFiles,
                 sum(fileSize) totalFileSize
            FROM DrainingDiskCopy
           GROUP BY fileSystem, status
        )
       GROUP BY fileSystem
    ) DDCS
   RIGHT JOIN DrainingFileSystem DFS
      ON DFS.fileSystem = DDCS.fileSystem
   INNER JOIN FileSystem FS
      ON DFS.fileSystem = FS.id
   INNER JOIN DiskServer DS
      ON FS.diskServer = DS.id
   INNER JOIN SvcClass SVC
      ON DFS.svcClass = SVC.id
   ORDER BY DS.name, FS.mountPoint;


/* SQL statement for the removeFailedDrainingTransfers procedure */
CREATE OR REPLACE PROCEDURE removeFailedDrainingTransfers(fsId IN NUMBER)
AS
  CURSOR c IS
    SELECT STDCRR.id, SR.id subrequest, STDCRR.client
      FROM StageDiskCopyReplicaRequest STDCRR, DrainingDiskCopy DDC,
           SubRequest SR
     WHERE STDCRR.sourceDiskCopy = DDC.diskCopy
       AND SR.request = STDCRR.id
       AND decode(DDC.status, 4, DDC.status, NULL) = 4  -- FAILED
       AND SR.status = 9  -- FAILED_FINISHED
       AND DDC.fileSystem = fsId;
  reqIds    "numList";
  clientIds "numList";
  srIds     "numList";
BEGIN
  -- Remove failed transfers requests from the StageDiskCopyReplicaRequest
  -- table. If we do not do this files which failed due to "Maximum number of
  -- attempts exceeded" cannot be resubmitted to the system.
  -- (see getBestDiskCopyToReplicate)
  LOOP
    OPEN c;
    FETCH c BULK COLLECT INTO reqIds, srIds, clientIds LIMIT 10000;
    -- Break out of the loop when the cursor returns no results
    EXIT WHEN reqIds.count = 0;
    -- Delete data
    FORALL i IN reqIds.FIRST .. reqIds.LAST
      DELETE FROM Id2Type WHERE id IN (reqIds(i), clientIds(i), srIds(i));
    FORALL i IN clientIds.FIRST .. clientIds.LAST
      DELETE FROM Client WHERE id = clientIds(i);
    FORALL i IN srIds.FIRST .. srIds.LAST
      DELETE FROM SubRequest WHERE id = srIds(i);
    CLOSE c;
  END LOOP;
  -- Delete all data related to the filesystem from the draining diskcopy table
  DELETE FROM DrainingDiskCopy
   WHERE fileSystem = fsId;
END;
/

/* Procedure responsible for stopping the draining process for a diskserver
 * or filesystem. In no filesystem is specified then all filesystems
 * associated to the diskserver will be stopped.
 */
CREATE OR REPLACE PROCEDURE stopDraining(inNodeName   IN VARCHAR,
                                         inMountPoint IN VARCHAR2 DEFAULT NULL,
                                         inRestart    IN NUMBER DEFAULT 0)
AS
  fsIds  "numList";
  unused NUMBER;
  mntPnt VARCHAR2(2048);
BEGIN
  -- Check that the nodename and mountpoint input options are valid
  SELECT FileSystem.id BULK COLLECT INTO fsIds
    FROM FileSystem, DiskServer
   WHERE FileSystem.diskServer = DiskServer.id
     AND (FileSystem.mountPoint = inMountPoint
      OR inMountPoint IS NULL)
     AND DiskServer.name = inNodeName;
  IF fsIds.COUNT = 0 THEN
    IF inMountPoint IS NULL THEN
      raise_application_error
        (-20019, 'Diskserver does not exist or has no mountpoints');
    ELSE
      raise_application_error
        (-20015, 'Diskserver and mountpoint does not exist');
    END IF;
  END IF;
  -- Update the filesystem entries to DELETING or RESTART depending in the
  -- inRestart option. The drainManager job will take care of actually doing
  -- the work.
  FOR i IN fsIds.FIRST .. fsIds.LAST
  LOOP
    -- Check to see if the mountpoint and diskserver combination allow for a
    -- draining operation to begin.
    BEGIN
      SELECT mountPoint INTO mntPnt
        FROM FileSystem WHERE id = fsIds(i);
      -- If restarting verify that the diskserver and filesystem are in a 
      -- valid state, code copied from startDraining.
      IF inRestart = 1 THEN
        SELECT FS.diskPool INTO unused
          FROM FileSystem FS, DiskServer DS
         WHERE FS.diskServer = DS.id
           AND FS.id = fsIds(i)
           AND (FS.status = 1 OR DS.status = 1)
           AND FS.status != 2
           AND DS.status != 2;
      END IF;
    EXCEPTION WHEN NO_DATA_FOUND THEN
      raise_application_error
        (-20116, 'Mountpoint: Restart not possible '||mntPnt||' is not in a DRAINING state or diskserver is DISABLED');
    END;
    UPDATE DrainingFileSystem
       SET status = decode(inRestart, 1, 7, 6)
     WHERE fileSystem = fsIds(i);
  END LOOP;
END;
/

/* Procedure responsible for starting the draining process for a diskserver
 * or filesystem. If no filesystem is specified then all filesystems
 * associated to the diskserver will be add to the list of filesystems to be
 * drained.
 */
CREATE OR REPLACE
PROCEDURE startDraining(inNodeName     IN VARCHAR,
                        inMountPoint   IN VARCHAR DEFAULT NULL,
                        inSvcClass     IN VARCHAR DEFAULT NULL,
                        inFileMask     IN NUMBER DEFAULT 1,
                        inAutoDelete   IN NUMBER DEFAULT 0,
                        inMaxTransfers IN NUMBER DEFAULT 50)
AS
  ret    NUMBER;
  fsIds  "numList";
  svcId  NUMBER;
  unused NUMBER;
  mntPnt VARCHAR2(2048);
BEGIN
  -- Check that the nodename and mountpoint input options are valid
  SELECT FileSystem.id BULK COLLECT INTO fsIds
    FROM FileSystem, DiskServer
   WHERE FileSystem.diskServer = Diskserver.id
     AND (FileSystem.mountPoint = inMountPoint
      OR inMountPoint IS NULL)
     AND DiskServer.name = inNodeName;
  IF fsIds.COUNT = 0 THEN
    IF inMountPoint IS NULL THEN
      raise_application_error
        (-20019, 'Diskserver does not exist or has no mountpoints');
    ELSE
      raise_application_error
        (-20015, 'Diskserver and mountpoint does not exist');
    END IF;
  END IF;
  -- Loop over the mountpoints extracted above and validate that the service
  -- class and status of the mountpoint are correct.
  FOR i IN fsIds.FIRST .. fsIds.LAST
  LOOP
    SELECT mountPoint INTO mntPnt
      FROM FileSystem WHERE id = fsIds(i);
    -- If no service class option is defined attempt to automatically find one.
    -- Note: this only works if the filesystem belongs to one and only one
    -- service class. If this is not the case the user must explicit provide
    -- one on the command line using the --svcclass option to the
    -- draindiskserver tool.
    IF inSvcClass IS NULL THEN
      BEGIN
        SELECT SvcClass.id INTO svcId
          FROM FileSystem, DiskPool2SvcClass, SvcClass
         WHERE FileSystem.diskPool = DiskPool2SvcClass.parent
           AND DiskPool2SvcClass.child = SvcClass.id
           AND FileSystem.id = fsIds(i);
      EXCEPTION
        WHEN TOO_MANY_ROWS THEN
          raise_application_error
            (-20101, 'Mountpoint: '||mntPnt||' belongs to multiple service classes, please specify which service class to use, using the --svcclass option');
        WHEN NO_DATA_FOUND THEN
          raise_application_error
            (-20120, 'Mountpoint: '||mntPnt||' does not belong to any service class');
      END;
    ELSE
      -- Check if the user supplied service class name exists
      ret := checkForValidSvcClass(inSvcClass, 0, 1);
      -- Check that the mountpoint belongs to the service class provided by the
      -- user. This check is necessary as we do not support the draining of a
      -- mountpoint to a service class which it is not already a member of.
      BEGIN
        SELECT SvcClass.id INTO SvcId
          FROM FileSystem, DiskPool2SvcClass, SvcClass
         WHERE FileSystem.diskPool = DiskPool2SvcClass.parent
           AND DiskPool2SvcClass.child = SvcClass.id
           AND FileSystem.id = fsIds(i)
           AND SvcClass.name = inSvcClass;
      EXCEPTION WHEN NO_DATA_FOUND THEN
        raise_application_error
          (-20117, 'Mountpoint: '||mntPnt||' does not belong to the '''||inSvcClass||''' service class');
      END;
    END IF;
    -- Check to see if the mountpoint and diskserver combination allow for a
    -- draining operation to begin.
    BEGIN
      SELECT FS.diskPool INTO unused
        FROM FileSystem FS, DiskServer DS
       WHERE FS.diskServer = DS.id
         AND FS.id = fsIds(i)
         AND (FS.status = 1 OR DS.status = 1)
         AND FS.status != 2
         AND DS.status != 2;
    EXCEPTION WHEN NO_DATA_FOUND THEN
      raise_application_error
        (-20116, 'Mountpoint: '||mntPnt||' is not in a DRAINING state or diskserver is DISABLED');
    END;
    -- Check to see if the mountpoint is already being drained. Note: we do not
    -- allow the resubmission of a mountpoint without a prior DELETION unless
    -- the previous process was not in a FAILED or COMPLETED state
    BEGIN
      SELECT fileSystem INTO unused
        FROM DrainingFileSystem
       WHERE fileSystem = fsIds(i)
         AND status NOT IN (4, 5);  -- FAILED, COMPLETED
      raise_application_error
        (-20118, 'Mountpoint: '||mntPnt||' is already being drained');
    EXCEPTION WHEN NO_DATA_FOUND THEN
      -- Cleanup
      removeFailedDrainingTransfers(fsIds(i));
      DELETE FROM DrainingFileSystem
       WHERE fileSystem = fsIds(i);
      -- Insert the new mountpoint into the list of those to be drained. The
      -- drainManager job will later pick it up and start the draining process.
      INSERT INTO DrainingFileSystem
        (userName, machine, creationTime, fileSystem, svcClass, autoDelete,
         fileMask, maxTransfers)
      VALUES
        (-- For the time being the draindiskserver command is distributed with
         -- the castor-dbtools package and uses the /etc/castor/ORASTAGERCONFIG
         -- password to connect to the database. As the file is only readable
         -- by the root user and the st group, the OS_USER will either be root
         -- or stage. This is not very interesting!!
         sys_context('USERENV', 'OS_USER'),
         sys_context('USERENV', 'HOST'),
         getTime(), fsIds(i), svcId, inAutoDelete, inFileMask, inMaxTransfers);
    END;
  END LOOP;
END;
/

/* Procedure responsible for managing the draining process */
CREATE OR REPLACE PROCEDURE drainManager AS
  fsIds "numList";
  dcIds "numList";
BEGIN
  -- Delete (and restart if necessary) the filesystems which are:
  --  A) in a DELETING state and have no transfers pending in the scheduler.
  --  B) are COMPLETED and older than 7 days.
  FOR A IN (SELECT fileSystem, status FROM DrainingFileSystem
             WHERE (NOT EXISTS
               (SELECT 'x' FROM DrainingDiskCopy
                 WHERE fileSystem = DrainingFileSystem.fileSystem
                   AND status = 3)  -- WAITD2D
               AND status IN (6, 7))  -- DELETING, RESTART
                OR (status = 5 AND lastUpdateTime < getTime() - (7 * 86400)))
  LOOP
    -- If the status is RESTART, reset the draining filesystem entry to
    -- its default values and set its status to CREATED, otherwise delete it!
    removeFailedDrainingTransfers(a.fileSystem);
    IF a.status = 7 THEN
      UPDATE DrainingFileSystem
         SET creationTime = getTime(), startTime = 0, lastUpdateTime = 0,
             status = 0, totalFiles = 0, totalBytes = 0
       WHERE fileSystem = a.fileSystem;        
    ELSE
      DELETE FROM DrainingFileSystem
       WHERE fileSystem = a.fileSystem;
    END IF;
  END LOOP;
  -- If the draining filesystem is in a DELETING state and there are still
  -- running transfers for which the diskcopy they were waiting on has
  -- disappeared delete them. SR #112993
  SELECT DDC.diskCopy BULK COLLECT INTO dcIds
    FROM DrainingDiskCopy DDC, DrainingFileSystem DFS
   WHERE DDC.fileSystem = DFS.fileSystem
     AND DDC.status = 3  -- WAITD2D
     AND DFS.status = 6  -- DELETING
     AND NOT EXISTS
       (SELECT 'x' FROM DiskCopy WHERE id = DDC.diskCopy);
  DELETE FROM DrainingDiskCopy
   WHERE status = 3
     AND diskCopy IN
       (SELECT /*+ CARDINALITY(dcIdTable 5) */ *
          FROM TABLE (dcIds) dcIdTable);
  -- Process filesystems which in a CREATED state
  UPDATE DrainingFileSystem
     SET status = 1  -- INITIALIZING
   WHERE status = 0  -- CREATED
  RETURNING fileSystem BULK COLLECT INTO fsIds;
  -- Commit, this isn't really necessary but its done so that the user gets
  -- feedback when listing the filesystems which are being drained.
  COMMIT;
  IF fsIds.COUNT = 0 THEN
    -- Shrink the DrainingDiskCopy to keep the number of blocks small
    EXECUTE IMMEDIATE 'ALTER TABLE DrainingDiskCopy SHRINK SPACE CASCADE';
    RETURN;  -- No results
  END IF;
  -- Create the DrainingDiskCopy snapshot
  INSERT /*+ APPEND */ INTO DrainingDiskCopy
    (fileSystem, diskCopy, creationTime, priority, fileSize)
      (SELECT /*+ index(DC I_DiskCopy_FileSystem) */
              DC.fileSystem, DC.id, DC.creationTime, DC.status,
              DC.diskCopySize
         FROM DiskCopy DC, DrainingFileSystem DFS
        WHERE DFS.fileSystem = DC.fileSystem
          AND DFS.fileSystem IN
            (SELECT /*+ CARDINALITY(fsIdTable 5) */ *
               FROM TABLE (fsIds) fsIdTable)
          AND ((DC.status = 0         AND DFS.fileMask = 0)    -- STAGED
           OR  (DC.status = decode(DC.status, 10, DC.status, NULL)
                AND DFS.fileMask = 1)                          -- CANBEMIGR
           OR  (DC.status IN (0, 10)  AND DFS.fileMask = 2))); -- ALL
  -- Regenerate the statistics for the DrainingDiskCopy table
  DBMS_STATS.GATHER_TABLE_STATS
    (OWNNAME          => sys_context('USERENV', 'CURRENT_SCHEMA'),
     TABNAME          =>'DRAININGDISKCOPY',
     ESTIMATE_PERCENT => 100,
     METHOD_OPT       => 'FOR ALL COLUMNS SIZE 100',
     NO_INVALIDATE    => FALSE,
     FORCE            => TRUE);
  -- Update the DrainingFileSystem counters
  FOR a IN (SELECT fileSystem, count(*) files, sum(DDC.fileSize) bytes
              FROM DrainingDiskCopy DDC
             WHERE DDC.fileSystem IN
               (SELECT /*+ CARDINALITY(fsIdTable 5) */ *
                  FROM TABLE (fsIds) fsIdTable)
               AND DDC.status = 0  -- CREATED
             GROUP BY DDC.fileSystem)
  LOOP
    UPDATE DrainingFileSystem
       SET totalFiles = a.files,
           totalBytes = a.bytes
     WHERE fileSystem = a.fileSystem;
  END LOOP;
  -- Update the filesystem entries to RUNNING
  UPDATE DrainingFileSystem
     SET status = 2,  -- RUNNING
         startTime = getTime(),
         lastUpdateTime = getTime()
   WHERE fileSystem IN
     (SELECT /*+ CARDINALITY(fsIdTable 5) */ *
        FROM TABLE (fsIds) fsIdTable);
  -- Start the process of draining the filesystems. For an explanation of the
  -- query refer to: "Creating N Copies of a Row":
  -- http://forums.oracle.com/forums/message.jspa?messageID=1953433#1953433
  FOR a IN ( SELECT fileSystem
               FROM DrainingFileSystem
              WHERE fileSystem IN
                (SELECT /*+ CARDINALITY(fsIdTable 5) */ *
                   FROM TABLE (fsIds) fsIdTable)
            CONNECT BY CONNECT_BY_ROOT fileSystem = fileSystem
                AND LEVEL <= maxTransfers
                AND PRIOR sys_guid() IS NOT NULL
              ORDER BY totalBytes ASC, fileSystem)
  LOOP
    drainFileSystem(a.fileSystem);
  END LOOP;
END;
/

/* Procedure responsible for processing the snapshot of files that need to be
 * replicated in order to drain a filesystem.
 */
CREATE OR REPLACE
PROCEDURE drainFileSystem(fsId IN NUMBER)
AS
  unused     NUMBER;
  res        NUMBER;
  dcId       NUMBER;
  cfId       NUMBER;
  svcId      NUMBER;
  ouid       NUMBER;
  ogid       NUMBER;
  autoDelete NUMBER;
  fileMask   NUMBER;
BEGIN
  -- As this procedure is called recursively we release the previous calls
  -- locks. This prevents the procedure from locking too many castorfile
  -- entries which could result in service degradation.
  COMMIT;
  -- Update the filesystems entry in the DrainingFileSystem table to indicate
  -- that activity is ongoing. If no entry exist then the filesystem is not
  -- under the control of the draining logic.
  svcId := 0;
  UPDATE DrainingFileSystem
     SET lastUpdateTime = getTime()
   WHERE fileSystem = fsId
     AND status = 2  -- RUNNING
  RETURNING svcclass, autoDelete, fileMask INTO svcId, autoDelete, fileMask;
  IF svcId = 0 THEN
    RETURN;  -- Do nothing
  END IF;
  -- Extract the next diskcopy to be processed.
  dcId := 0;
  UPDATE DrainingDiskCopy
     SET status = 2  -- PROCESSING
   WHERE diskCopy = (
     SELECT diskCopy FROM (
       SELECT DDC.diskCopy
         FROM DrainingDiskCopy DDC
        WHERE DDC.fileSystem = fsId
          AND DDC.status IN (0, 1)  -- CREATED, RESTARTED
        ORDER BY DDC.priority DESC, DDC.creationTime ASC)
     WHERE rownum < 2)
  RETURNING diskCopy INTO dcId;
  IF dcId = 0 THEN
    -- If there are no transfers outstanding related to the draining process we
    -- can update the filesystem entry in the DrainingFileSystem table for the
    -- last time.
    UPDATE DrainingFileSystem
       SET status = decode((SELECT count(*)
                              FROM DrainingDiskCopy
                             WHERE status = 4  -- FAILED
                               AND fileSystem = fsId), 0, 5, 4),
           lastUpdateTime = getTime()
     WHERE fileSystem = fsId
       -- Check to make sure there are no transfers outstanding.
       AND (SELECT count(*) FROM DrainingDiskCopy
             WHERE status = 3  -- RUNNING
               AND fileSystem = fsId) = 0;
    RETURN;  -- Out of work!
  END IF;
  -- The DrainingDiskCopy table essentially contains a snapshot of the work
  -- that needs to be done at the time the draining process was initiated for a
  -- filesystem. As a result, the diskcopy id previously extracted may no longer
  -- be valid as the snapshot information is outdated. Here we simply verify
  -- that the diskcopy is still what we expected.
  BEGIN
    -- Determine the castorfile id
    SELECT castorFile INTO cfId FROM DiskCopy WHERE id = dcId;
    -- Lock the castorfile
    SELECT id INTO cfId FROM CastorFile
     WHERE id = cfId FOR UPDATE;
    -- Check that the status of the diskcopy matches what was specified in the
    -- filemask for the filesystem. If the diskcopy is not in the expected
    -- status then it is no longer a candidate to be replicated.
    SELECT ownerUid, ownerGid INTO ouid, ogid
      FROM DiskCopy
     WHERE id = dcId
       AND ((status = 0        AND fileMask = 0)    -- STAGED
        OR  (status = 10       AND fileMask = 1)    -- CANBEMIGR
        OR  (status IN (0, 10) AND fileMask = 2));  -- ALL
  EXCEPTION WHEN NO_DATA_FOUND THEN
    -- The diskcopy no longer exists or is not of interest so we delete it from
    -- the snapshot of files to be processed.
    DELETE FROM DrainingDiskCopy
      WHERE diskCopy = dcId AND fileSystem = fsId;
    drainFileSystem(fsId);
    RETURN;  -- No replication required
  END;
  -- Just because the file was listed in the snapshot doesn't mean that it must
  -- be replicated! Here we check to see if the file is available on another
  -- diskserver in the target service class and if enough copies of the file
  -- are available.
  --
  -- The decode logic used here essentially states:
  --   If replication on close is enabled and there are enough copies available
  --   to satisfy the maxReplicaNb on the service class, then do not replicate.
  -- Otherwise,
  --   If replication on close is disabled and a copy exists elsewhere, then do
  --   do not replicate. All other cases result in triggering replication.
  --
  -- Notes:
  -- - The check to see if we have enough copies online when replication on
  --   close is enabled will effectively rebalance the pool!
  SELECT decode(replicateOnClose, 1,
         decode((available - maxReplicaNb) -
                 abs(available - maxReplicaNb), 0, 0, 1),
         decode(sign(available), 1, 0, 1)) INTO res
    FROM (
      SELECT count(*) available
        FROM DiskCopy, FileSystem, DiskServer, DiskPool2SvcClass
       WHERE DiskCopy.fileSystem = FileSystem.id
         AND DiskCopy.castorFile = cfId
         AND DiskCopy.id != dcId
         AND DiskCopy.status IN (0, 10)  -- STAGED, CANBEMIGR
         AND FileSystem.diskPool = DiskPool2SvcClass.parent
         AND DiskPool2SvcClass.child = svcId
         AND FileSystem.status IN (0, 1)  -- PRODUCTION, DRAINING
         AND FileSystem.diskServer = DiskServer.id
         AND DiskServer.status IN (0, 1)  -- PRODUCTION, DRAINING
     ) results, SvcClass
   WHERE SvcClass.id = svcId;
  IF res = 0 THEN
    -- No replication is required, there are enough copies of the file to
    -- satisfy the requirements of the target service class.
    IF autoDelete = 1 THEN
      -- Invalidate the diskcopy so that it can be garbage collected.
      UPDATE DiskCopy
         SET status = 7,  -- INVALID
             gctype = 3   -- Draining filesystem
       WHERE id = dcId AND fileSystem = fsId;
    END IF;
    -- Delete the diskcopy from the snapshot as no action is required.
    DELETE FROM DrainingDiskCopy
     WHERE diskCopy = dcId AND fileSystem = fsId;
    drainFileSystem(fsId);
    RETURN;
  END IF;
  -- Check to see if there is already an outstanding request to replicate the
  -- castorfile to the target service class. If so, we wait on that replication
  -- to complete. This avoids having multiple requests to replicate the same
  -- file to the same target service class multiple times.
  BEGIN
    SELECT DiskCopy.id INTO res
      FROM DiskCopy, StageDiskCopyReplicaRequest
     WHERE StageDiskCopyReplicaRequest.destDiskCopy = DiskCopy.id
       AND StageDiskCopyReplicaRequest.svcclass = svcId
       AND DiskCopy.castorFile = cfId
       AND DiskCopy.status = 1  -- WAITDISK2DISKCOPY
       AND rownum < 2;
    IF res > 0 THEN
      -- Wait on another replication to complete.
      UPDATE DrainingDiskCopy
         SET status = 3,  -- WAITD2D
             parent = res
       WHERE diskCopy = dcId AND fileSystem = fsId;
       RETURN;
    END IF;
  EXCEPTION WHEN NO_DATA_FOUND THEN
    NULL;  -- No pending replications running
  END;
  -- If we have attempted to replicate the file more than 10 times already then
  -- give up! The error will be exposed later to an administrator for manual
  -- corrective action.
  SELECT count(*) INTO res
    FROM StageDiskCopyReplicaRequest R, SubRequest
   WHERE SubRequest.request = R.id
     AND R.sourceDiskCopy = dcId
     AND SubRequest.status = 9; -- FAILED_FINISHED
  IF res >= 10 THEN
    UPDATE DrainingDiskCopy
       SET status = 4,
           comments = 'Maximum number of attempts exceeded'
     WHERE diskCopy = dcId AND fileSystem = fsId;
    drainFileSystem(fsId);
    RETURN;  -- Try again
  END IF;
  -- Trigger a replication request for the file
  createDiskCopyReplicaRequest(0, dcId, svcId, svcId, ouid, ogid);
  -- Update the status of the file
  UPDATE DrainingDiskCopy SET status = 3  -- WAITD2D
   WHERE diskCopy = dcId AND fileSystem = fsId;
END;
/


/* attach drive request to tape */


create or replace
PROCEDURE tg_attachDriveReqToTape( tapeRequestId IN NUMBER,
                                                       inputVdqmId IN NUMBER, 
                                                       inputDgn IN VARCHAR2, 
                                                       inputLabel IN VARCHAR2,
                                                       inputDensity IN VARCHAR2) AS
BEGIN

-- update tapegatewayrequest which have been submitted to vdqm =>  WAITING_TAPESERVER
    UPDATE TapeGatewayRequest 
      SET status=2, lastvdqmpingtime=gettime(), starttime= gettime(), vdqmvolreqid=inputVdqmId
      WHERE id=tapeRequestId; -- these are the ones waiting for vdqm to be sent again

-- update stream for migration    
    UPDATE Stream 
      SET status=1 
      WHERE id = 
        (select streammigration FROM TapeGatewayRequest WHERE accessmode=1 AND  id=tapeRequestId);  

-- update tape for migration and recall    

    UPDATE Tape 
      SET status=2, dgn=inputDgn, label=inputLabel, density= inputDensity 
      WHERE id = 
        (SELECT tape 
          FROM Stream, TapeGatewayRequest 
          WHERE stream.id = tapegatewayrequest.streammigration 
          AND tapegatewayrequest.accessmode=1 
          AND tapegatewayrequest.id=tapeRequestId) 
      OR id =
        (SELECT taperecall 
          FROM TapeGatewayRequest  
          WHERE accessmode=0 
          AND  id=tapeRequestId);  
  COMMIT;
END;
/
        
/* attach the tapes to the streams  */

CREATE OR REPLACE PROCEDURE tg_attachTapesToStreams (startFseqs IN castor."cnumList",
                                                     strIds IN castor."cnumList", 
                                                     tapeVids IN castor."strList") AS
  CONSTRAINT_VIOLATED EXCEPTION;
  PRAGMA EXCEPTION_INIT(CONSTRAINT_VIOLATED, -1);
  tapeId NUMBER;

BEGIN
 FOR i IN strIds.FIRST .. strIds.LAST LOOP
      tapeId:=NULL;
      -- update tapegatewayrequest
      UPDATE TapeGatewayRequest 
        SET status=1, lastfseq= startfseqs(i) 
        WHERE streammigration= strids(i); -- TO_BE_SENT_TO_VDQM
      
      UPDATE Tape SET stream=strids(i), status=2 
        WHERE tpmode=1 
        AND vid=tapeVids(i) 
        RETURNING id INTO tapeId;
 
      IF tapeId IS NULL THEN 
        BEGIN

        -- insert a tape if it is not already in the database
          INSERT INTO Tape 
               (vid,side,tpmode,errmsgtxt,errorcode,severity,vwaddress,id,stream,status) 
               VALUES (tapeVids(i),0,1,null,0,0,null,ids_seq.nextval,0,2) 
               RETURNING id INTO tapeId;
          INSERT INTO id2type (id,type) values (tapeId,29);
      
        EXCEPTION WHEN CONSTRAINT_VIOLATED THEN

       -- retry the select since a creation was done in between
          UPDATE Tape SET stream=strids(i), status=2
            WHERE tpmode=1 
            AND vid=tapeVids(i) 
            RETURNING id INTO tapeId;
        
        END;

      END IF;
  
    -- update the stream
      UPDATE Stream SET tape=tapeId 
        WHERE Stream.id=strIds(i); 

 END LOOP;
 COMMIT;
END;
/

/* update the db when a tape session is ended */

create or replace
PROCEDURE tg_endTapeSession ( transactionId IN NUMBER,
                                                inputErrorCode IN INTEGER) AS
  CONSTRAINT_VIOLATED EXCEPTION;
  PRAGMA EXCEPTION_INIT(CONSTRAINT_VIOLATED, -02292);

  tpId NUMBER;
  trId NUMBER;
  strId NUMBER;
  reqMode INTEGER;
  segNum INTEGER;
  cfId NUMBER;
  tcIds "numList";
  srIds "numList";

BEGIN
  BEGIN
  -- take lock on taperequest
   SELECT id,taperecall, streammigration,accessmode
        INTO trId,tpId, strid, reqMode 
        FROM TapeGatewayRequest 
        WHERE vdqmvolreqid = transactionId FOR UPDATE;

   -- delete tapesubrequest
    DELETE FROM TapeGatewaySubRequest
        WHERE request=trId
        RETURNING tapecopy,id bulk collect
        INTO tcIds, srIds;

    FORALL i in srIds.FIRST .. srIds.LAST
     DELETE FROM id2type WHERE id=srIds(i);
     
  -- delete taperequest after the subrequest to avouid constraint violation  
     
    DELETE FROM TapeGatewayRequest
        WHERE id=trId;
    DELETE FROM id2type WHERE id=trId;

    IF reqMode = 0 THEN
     -- read
      SELECT id INTO tpId
        FROM Tape
        WHERE id =tpId
        FOR UPDATE;

       IF inputErrorCode != 0 THEN

          -- if a failure is reported
          -- fail all the segments
          FORALL i in tcIds.FIRST .. tcIds.LAST
            UPDATE Segment SET status=6
              WHERE copy = tcIds(i);

          -- mark tapecopies as  REC_RETRY
          FORALL i in tcIds.FIRST .. tcIds.LAST
            UPDATE TapeCopy  SET status=8, errorcode=inputErrorCode
              WHERE id=tcIds(i);

       END IF;

       -- resurrect lost tapesubrequest
       UPDATE Segment SET status=0
         WHERE status = 7
         AND tape=tpId;

       -- check if there is work for this tape
       SELECT count(*) INTO segNum
         FROM segment
         WHERE tape=tpId
         AND status=0;
       -- still segments in zero ... to be restarted

       IF segNum > 0 THEN
         UPDATE Tape SET status = 8
           WHERE id=tpId; --WAIT POLICY for rechandler
       ELSE
         UPDATE Tape SET status = 0
           WHERE id=tpId; --UNUSED
       END IF;

    ELSE

      -- write
      -- reset stream
       BEGIN
         DELETE FROM Stream WHERE id = strId;
         DELETE FROM id2type WHERE id=strId;

       EXCEPTION WHEN CONSTRAINT_VIOLATED THEN
       -- constraint violation and we cannot delete the stream because there is an entry in Stream2TapeCopy
         UPDATE Stream SET status = 6, tape = null, lastFileSystemChange = null
           WHERE id = strId;
       END;
       -- reset tape
       UPDATE Tape SET status=0, stream = null WHERE stream = strId;

       IF inputErrorCode != 0 THEN
          -- if a failure is reported
          --retry MIG_RETRY
          FORALL i in tcIds.FIRST .. tcIds.LAST
            UPDATE TapeCopy  SET status=9, errorcode=inputErrorCode, nbretry=0
              WHERE id=tcIds(i);
       ELSE
         -- just resurrect them if they were lost
          FORALL i in tcIds.FIRST .. tcIds.LAST
           UPDATE TapeCopy  SET status=1
              WHERE id=tcIds(i) AND status=3;
       END IF;


    END IF;

    EXCEPTION WHEN  NO_DATA_FOUND THEN
      null;
    END;

    COMMIT;
END;
/


/* mark a migration or recall as failed saving in the db the error code associated with the failure */

CREATE OR REPLACE PROCEDURE tg_failFileTransfer(transactionId IN NUMBER,
                                                inputFileId IN NUMBER,
                                                inputNsHost IN VARCHAR2,
                                                inputFseq IN INTEGER, 
                                                inputErrorCode IN INTEGER)  AS
 cfId NUMBER;
 trId NUMBER;
 strId NUMBER;
 tpId NUMBER;
 tcId NUMBER;
 srId NUMBER;
 outMode INTEGER;

BEGIN
 
  SELECT accessmode, id, taperecall, streammigration INTO outMode, trId, tpId, strId 
    FROM  TapeGatewayRequest 
    WHERE transactionid = vdqmvolreqid 
    FOR UPDATE;
   
  SELECT id INTO cfId 
    FROM CastorFile 
    WHERE fileid=inputFileId
    AND nshost=inputNsHost 
    FOR UPDATE;

 -- delete tape subrequest
  DELETE FROM TapegatewaySubRequest 
    WHERE fseq=inputFseq 
    AND request=trId 
    RETURNING tapecopy,id  INTO tcId, srId;

  DELETE FROM id2type WHERE id=srId;
  
  IF outmode = 0 THEN
     -- read
     -- retry REC_RETRY 
     -- failed the segment on that tape
     UPDATE Segment SET status=6, severity=inputErrorCode, errorCode=-1 
       WHERE fseq= inputfseq 
       AND tape=tpId 
       RETURNING copy INTO tcId;
 
     -- mark tapecopy as REC_RETRY
     UPDATE TapeCopy  SET status=8, errorcode=inputErrorCode 
       WHERE id=tcId;   
  
  ELSE 
     -- write
     SELECT Tape.id INTO tpId 
       FROM Tape, Stream 
       WHERE Tape.id = Stream.tape
       AND Stream.id = strId;
    
     --retry MIG_RETRY
     -- mark tapecopy as MIG_RETRY
     UPDATE TapeCopy  SET status=9, errorcode=inputErrorCode 
       WHERE id=tcId; 
  
  END IF;  
EXCEPTION WHEN  NO_DATA_FOUND THEN
    null;
END;
/

/* retrieve from the db all the tapecopies that faced a failure for migration */

CREATE OR REPLACE PROCEDURE  tg_getFailedMigrations(tcs OUT castor.TapeCopy_Cur) AS
  ids "numList";
BEGIN
  -- get TAPECOPY_MIG_RETRY
  OPEN tcs FOR
    SELECT copynb, id, castorFile, status, errorCode, nbretry, missingCopies 
      FROM TapeCopy
      WHERE status=9 
      AND ROWNUM < 1000 
      FOR UPDATE SKIP LOCKED; 
END;
/


/* retrieve from the db all the tapecopies that faced a failure for recall */

CREATE OR REPLACE PROCEDURE  tg_getFailedRecalls(tcs OUT castor.TapeCopy_Cur) AS
  ids "numList";
BEGIN
  -- get TAPECOPY_REC_RETRY
  OPEN tcs FOR
    SELECT copynb, id, castorFile, status, errorCode, nbretry, missingCopies
     FROM TapeCopy
     WHERE status=8 
     AND ROWNUM < 1000 
     FOR UPDATE SKIP LOCKED; 
END;
/

/* default migration candidate selection policy */

create or replace
PROCEDURE tg_defaultMigrSelPolicy(streamId IN INTEGER,
                                  diskServerName OUT NOCOPY VARCHAR2,
                                  mountPoint OUT NOCOPY VARCHAR2,
                                  path OUT NOCOPY VARCHAR2,
                                  dci OUT INTEGER,
                                  castorFileId OUT INTEGER,
                                  fileId OUT INTEGER,
                                  nsHost OUT NOCOPY VARCHAR2, 
                                  fileSize OUT INTEGER,
                                  tapeCopyId OUT INTEGER, 
                                  lastUpdateTime OUT INTEGER) AS
  fileSystemId INTEGER := 0;
  diskServerId NUMBER;
  lastFSChange NUMBER;
  lastFSUsed NUMBER;
  lastButOneFSUsed NUMBER;
  findNewFS NUMBER := 1;
  nbMigrators NUMBER := 0;
  unused NUMBER;
  LockError EXCEPTION;
  PRAGMA EXCEPTION_INIT (LockError, -54);
BEGIN
  tapeCopyId := 0;
  -- First try to see whether we should resuse the same filesystem as last time
  SELECT lastFileSystemChange, lastFileSystemUsed, lastButOneFileSystemUsed
    INTO lastFSChange, lastFSUsed, lastButOneFSUsed
    FROM Stream WHERE id = streamId;
  IF getTime() < lastFSChange + 900 THEN
    SELECT (SELECT count(*) FROM stream WHERE lastFileSystemUsed = lastButOneFSUsed) +
           (SELECT count(*) FROM stream WHERE lastButOneFileSystemUsed = lastButOneFSUsed)
      INTO nbMigrators FROM DUAL;
    -- only go if we are the only migrator on the box
    IF nbMigrators = 1 THEN
      BEGIN
        -- check states of the diskserver and filesystem and get mountpoint and diskserver name
        SELECT name, mountPoint, FileSystem.id INTO diskServerName, mountPoint, fileSystemId
          FROM FileSystem, DiskServer
         WHERE FileSystem.diskServer = DiskServer.id
           AND FileSystem.id = lastButOneFSUsed
           AND FileSystem.status IN (0, 1)  -- PRODUCTION, DRAINING
           AND DiskServer.status IN (0, 1); -- PRODUCTION, DRAINING
        -- we are within the time range, so we try to reuse the filesystem
        SELECT /*+ FIRST_ROWS(1)  LEADING(D T ST) */
               D.path, D.id, D.castorfile, T.id
          INTO path, dci, castorFileId, tapeCopyId
          FROM DiskCopy D, TapeCopy T, Stream2TapeCopy ST
         WHERE decode(D.status, 10, D.status, NULL) = 10 -- CANBEMIGR
           AND D.filesystem = lastButOneFSUsed
           AND ST.parent = streamId
           AND T.status = 2 -- WAITINSTREAMS
           AND ST.child = T.id
           AND T.castorfile = D.castorfile
           AND ROWNUM < 2 FOR UPDATE OF t.id NOWAIT;
        SELECT CastorFile.FileId, CastorFile.NsHost, CastorFile.FileSize, CastorFile.lastUpdateTime
          INTO fileId, nsHost, fileSize, lastUpdateTime
          FROM CastorFile
         WHERE Id = castorFileId;
        -- we found one, no need to go for new filesystem
        findNewFS := 0;
      EXCEPTION WHEN NO_DATA_FOUND THEN
        -- found no tapecopy or diskserver, filesystem are down. We'll go through the normal selection
        NULL;
      WHEN LockError THEN
      -- We have observed ORA-00054 errors (resource busy and acquire with NOWAIT) even with
      -- the SKIP LOCKED clause. This is a workaround to ignore the error until we understand
      -- what to do, another thread will pick up the request so we don't do anything.
        NULL;
      END;
    END IF;
  END IF;
  IF findNewFS = 1 THEN
    FOR f IN (
    SELECT FileSystem.id AS FileSystemId, DiskServer.id AS DiskServerId, DiskServer.name, FileSystem.mountPoint
       FROM Stream, SvcClass2TapePool, DiskPool2SvcClass, FileSystem, DiskServer
      WHERE Stream.id = streamId
        AND Stream.TapePool = SvcClass2TapePool.child
        AND SvcClass2TapePool.parent = DiskPool2SvcClass.child
        AND DiskPool2SvcClass.parent = FileSystem.diskPool
        AND FileSystem.diskServer = DiskServer.id
        AND FileSystem.status IN (0, 1) -- PRODUCTION, DRAINING
        AND DiskServer.status IN (0, 1) -- PRODUCTION, DRAINING
      ORDER BY -- first prefer diskservers where no migrator runs and filesystems with no recalls
               DiskServer.nbMigratorStreams ASC, FileSystem.nbRecallerStreams ASC,
               -- then order by rate as defined by the function
               fileSystemRate(FileSystem.readRate, FileSystem.writeRate, FileSystem.nbReadStreams, FileSystem.nbWriteStreams,
                              FileSystem.nbReadWriteStreams, FileSystem.nbMigratorStreams, FileSystem.nbRecallerStreams) DESC,
               -- finally use randomness to avoid preferring always the same FS
               DBMS_Random.value) LOOP
       BEGIN
         -- lock the complete diskServer as we will update all filesystems
         SELECT id INTO unused FROM DiskServer WHERE id = f.DiskServerId FOR UPDATE NOWAIT;
         SELECT /*+ FIRST_ROWS(1) LEADING(D T StT C) */
                f.diskServerId, f.name, f.mountPoint, f.fileSystemId, D.path, D.id, D.castorfile, C.fileId, C.nsHost, C.fileSize, T.id, C.lastUpdateTime
           INTO diskServerId, diskServerName, mountPoint, fileSystemId, path, dci, castorFileId, fileId, nsHost, fileSize, tapeCopyId, lastUpdateTime
           FROM DiskCopy D, TapeCopy T, Stream2TapeCopy StT, Castorfile C
          WHERE decode(D.status, 10, D.status, NULL) = 10 -- CANBEMIGR
            AND D.filesystem = f.fileSystemId
            AND StT.parent = streamId
            AND T.status = 2 -- WAITINSTREAMS
            AND StT.child = T.id
            AND T.castorfile = D.castorfile
            AND C.id = D.castorfile
            AND ROWNUM < 2 FOR UPDATE OF t.id NOWAIT;
         -- found something on this filesystem, no need to go on
         diskServerId := f.DiskServerId;
         fileSystemId := f.fileSystemId;
         EXIT;
       EXCEPTION WHEN NO_DATA_FOUND OR LockError THEN
         -- either the filesystem is already locked or we found nothing,
         -- let's go to the next one
         NULL;
       END;
    END LOOP;
  END IF;

  IF tapeCopyId = 0 THEN
    -- Nothing found, return; locks will be released by the caller
    RETURN;
  END IF;
  
  IF findNewFS = 1 THEN
    UPDATE Stream
       SET lastFileSystemUsed = fileSystemId,
           lastButOneFileSystemUsed = lastFileSystemUsed,
           lastFileSystemChange = getTime()
     WHERE id = streamId;
  END IF;

  -- Update Filesystem state
  updateFSMigratorOpened(diskServerId, fileSystemId, 0);
END;
/

/* drain disk migration candidate selection policy */

create or replace PROCEDURE tg_drainDiskMigrSelPolicy(streamId IN INTEGER,
                                    diskServerName OUT NOCOPY VARCHAR2, 
                                    mountPoint OUT NOCOPY VARCHAR2,
                                    path OUT NOCOPY VARCHAR2,
                                    dci OUT INTEGER,
                                    castorFileId OUT INTEGER,
                                    fileId OUT INTEGER,
                                    nsHost OUT NOCOPY VARCHAR2,
                                    fileSize OUT INTEGER,
                                    tapeCopyId OUT INTEGER,
                                    lastUpdateTime OUT INTEGER) AS
  fileSystemId INTEGER := 0;
  diskServerId NUMBER;
  fsDiskServer NUMBER;
  lastFSChange NUMBER;
  lastFSUsed NUMBER;
  lastButOneFSUsed NUMBER;
  findNewFS NUMBER := 1;
  nbMigrators NUMBER := 0;
  unused NUMBER;
  LockError EXCEPTION;
  PRAGMA EXCEPTION_INIT (LockError, -54);
BEGIN
  tapeCopyId := 0;
  -- First try to see whether we should resuse the same filesystem as last time
  SELECT lastFileSystemChange, lastFileSystemUsed, lastButOneFileSystemUsed
    INTO lastFSChange, lastFSUsed, lastButOneFSUsed
    FROM Stream WHERE id = streamId;
  IF getTime() < lastFSChange + 1800 THEN
    SELECT (SELECT count(*) FROM stream WHERE lastFileSystemUsed = lastFSUsed)
      INTO nbMigrators FROM DUAL;
    -- only go if we are the only migrator on the box
    IF nbMigrators = 1 THEN
      BEGIN
        -- check states of the diskserver and filesystem and get mountpoint and diskserver name
        SELECT diskserver.id, name, mountPoint, FileSystem.id INTO diskServerId, diskServerName, mountPoint, fileSystemId
          FROM FileSystem, DiskServer
         WHERE FileSystem.diskServer = DiskServer.id
           AND FileSystem.id = lastFSUsed
           AND FileSystem.status IN (0, 1)  -- PRODUCTION, DRAINING
           AND DiskServer.status IN (0, 1); -- PRODUCTION, DRAINING
        -- we are within the time range, so we try to reuse the filesystem
        
        SELECT /*+ ORDERED USE_NL(D T) INDEX(T I_TapeCopy_CF_Status_2) INDEX(ST I_Stream2TapeCopy_PC) */
              D.path, D.diskcopy_id, D.castorfile, T.id INTO path, dci, castorFileId, tapeCopyId
          FROM (SELECT /*+ INDEX(DK I_DiskCopy_FS_Status_10) */
                             DK.path path, DK.id diskcopy_id, DK.castorfile
                  FROM DiskCopy DK
                  WHERE decode(DK.status, 10, DK.status, NULL) = 10 -- CANBEMIGR
                  AND DK.filesystem = lastFSUsed)  D, TapeCopy T, Stream2TapeCopy ST
          WHERE T.castorfile = D.castorfile
          AND ST.child = T.id
          AND ST.parent = streamId
          AND decode(T.status, 2, T.status, NULL) = 2 -- WAITINSTREAMS
          AND ROWNUM < 2 FOR UPDATE OF T.id NOWAIT;   
        
        SELECT  C.fileId, C.nsHost, C.fileSize,  C.lastUpdateTime
          INTO  fileId, nsHost, fileSize, lastUpdateTime
          FROM  castorfile C
          WHERE castorfileId = C.id;
         
        -- we found one, no need to go for new filesystem
        findNewFS := 0;
      EXCEPTION WHEN NO_DATA_FOUND OR LockError THEN
        -- found no tapecopy or diskserver, filesystem are down. We'll go through the normal selection
        NULL;
      END;
    END IF;
  END IF;
  IF findNewFS = 1 THEN
    -- We try first to reuse the diskserver of the lastFSUsed, even if we change filesystem
    FOR f IN (
      SELECT FileSystem.id AS FileSystemId, DiskServer.id AS DiskServerId, DiskServer.name, FileSystem.mountPoint
        FROM FileSystem, DiskServer
       WHERE FileSystem.diskServer = DiskServer.id
         AND FileSystem.status IN (0, 1) -- PRODUCTION, DRAINING
         AND DiskServer.status IN (0, 1) -- PRODUCTION, DRAINING
         AND DiskServer.id = lastButOneFSUsed) LOOP
       BEGIN
         -- lock the complete diskServer as we will update all filesystems
         SELECT id INTO unused FROM DiskServer WHERE id = f.DiskServerId FOR UPDATE NOWAIT;
         SELECT /*+ FIRST_ROWS(1) LEADING(D T StT C) */
                f.diskServerId, f.name, f.mountPoint, f.fileSystemId, D.path, D.id, D.castorfile, C.fileId, C.nsHost, C.fileSize, T.id, C.lastUpdateTime
           INTO diskServerId, diskServerName, mountPoint, fileSystemId, path, dci, castorFileId, fileId, nsHost, fileSize, tapeCopyId, lastUpdateTime
           FROM DiskCopy D, TapeCopy T, Stream2TapeCopy StT, Castorfile C
          WHERE decode(D.status, 10, D.status, NULL) = 10 -- CANBEMIGR
            AND D.filesystem = f.fileSystemId
            AND StT.parent = streamId
            AND T.status = 2 -- WAITINSTREAMS
            AND StT.child = T.id
            AND T.castorfile = D.castorfile
            AND C.id = D.castorfile
            AND ROWNUM < 2 FOR UPDATE OF T.id NOWAIT;
         -- found something on this filesystem, no need to go on
         diskServerId := f.DiskServerId;
         fileSystemId := f.fileSystemId;
         EXIT;
       EXCEPTION WHEN NO_DATA_FOUND OR LockError THEN
         -- either the filesystem is already locked or we found nothing,
         -- let's go to the next one
         NULL;
       END;
    END LOOP;
  END IF;
  IF tapeCopyId = 0 THEN
    -- Then we go for all potential filesystems. Note the duplication of code, due to the fact that ORACLE cannot order unions
    FOR f IN (
      SELECT FileSystem.id AS FileSystemId, DiskServer.id AS DiskServerId, DiskServer.name, FileSystem.mountPoint
        FROM Stream, SvcClass2TapePool, DiskPool2SvcClass, FileSystem, DiskServer
       WHERE Stream.id = streamId
         AND Stream.TapePool = SvcClass2TapePool.child
         AND SvcClass2TapePool.parent = DiskPool2SvcClass.child
         AND DiskPool2SvcClass.parent = FileSystem.diskPool
         AND FileSystem.diskServer = DiskServer.id
         AND FileSystem.status IN (0, 1) -- PRODUCTION, DRAINING
         AND DiskServer.status IN (0, 1) -- PRODUCTION, DRAINING
       ORDER BY -- first prefer diskservers where no migrator runs and filesystems with no recalls
                DiskServer.nbMigratorStreams ASC, FileSystem.nbRecallerStreams ASC,
                -- then order by rate as defined by the function
                fileSystemRate(FileSystem.readRate, FileSystem.writeRate, FileSystem.nbReadStreams, FileSystem.nbWriteStreams,
                               FileSystem.nbReadWriteStreams, FileSystem.nbMigratorStreams, FileSystem.nbRecallerStreams) DESC,
                -- finally use randomness to avoid preferring always the same FS
                DBMS_Random.value) LOOP
       BEGIN
         -- lock the complete diskServer as we will update all filesystems
         SELECT id INTO unused FROM DiskServer WHERE id = f.DiskServerId FOR UPDATE NOWAIT;
         SELECT /*+ FIRST_ROWS(1) LEADING(D T StT C) */
                f.diskServerId, f.name, f.mountPoint, f.fileSystemId, D.path, D.id, D.castorfile, C.fileId, C.nsHost, C.fileSize, T.id, C.lastUpdateTime
           INTO diskServerId, diskServerName, mountPoint, fileSystemId, path, dci, castorFileId, fileId, nsHost, fileSize, tapeCopyId, lastUpdateTime
           FROM DiskCopy D, TapeCopy T, Stream2TapeCopy StT, Castorfile C
          WHERE decode(D.status, 10, D.status, NULL) = 10 -- CANBEMIGR
            AND D.filesystem = f.fileSystemId
            AND StT.parent = streamId
            AND T.status = 2 -- WAITINSTREAMS
            AND StT.child = T.id
            AND T.castorfile = D.castorfile
            AND C.id = D.castorfile
            AND ROWNUM < 2 FOR UPDATE OF T.id NOWAIT;
         -- found something on this filesystem, no need to go on
         diskServerId := f.DiskServerId;
         fileSystemId := f.fileSystemId;
         EXIT;
       EXCEPTION WHEN NO_DATA_FOUND OR LockError THEN
         -- either the filesystem is already locked or we found nothing,
         -- let's go to the next one
         NULL;
       END;
    END LOOP;
  END IF;

  IF tapeCopyId = 0 THEN
    -- Nothing found, return; locks will be released by the caller
    RETURN;
  END IF;
  
  IF findNewFS = 1 THEN
    UPDATE Stream
       SET lastFileSystemUsed = fileSystemId,
           lastButOneFileSystemUsed = lastFileSystemUsed,
           lastFileSystemChange = getTime()
     WHERE id = streamId;
  END IF;

  -- Update Filesystem state
  updateFSMigratorOpened(diskServerId, fileSystemId, 0);
END;
/


/* repack migration candidate selection policy */

create or replace
PROCEDURE tg_repackMigrSelPolicy(streamId IN INTEGER,
                                 diskServerName OUT NOCOPY VARCHAR2,
                                 mountPoint OUT NOCOPY VARCHAR2,
                                 path OUT NOCOPY VARCHAR2, dci OUT INTEGER,
                                 castorFileId OUT INTEGER, fileId OUT INTEGER,
                                 nsHost OUT NOCOPY VARCHAR2,
                                 fileSize OUT INTEGER,
                                 tapeCopyId OUT INTEGER,
                                 lastUpdateTime OUT INTEGER) AS
  fileSystemId INTEGER := 0;
  diskServerId NUMBER;
  lastFSChange NUMBER;
  lastFSUsed NUMBER;
  lastButOneFSUsed NUMBER;
  nbMigrators NUMBER := 0;
  unused NUMBER;
  LockError EXCEPTION;
  PRAGMA EXCEPTION_INIT (LockError, -54);
BEGIN
  tapeCopyId := 0;
  FOR f IN (
    SELECT FileSystem.id AS FileSystemId, DiskServer.id AS DiskServerId, DiskServer.name, FileSystem.mountPoint
       FROM Stream, SvcClass2TapePool, DiskPool2SvcClass, FileSystem, DiskServer
      WHERE Stream.id = streamId
        AND Stream.TapePool = SvcClass2TapePool.child
        AND SvcClass2TapePool.parent = DiskPool2SvcClass.child
        AND DiskPool2SvcClass.parent = FileSystem.diskPool
        AND FileSystem.diskServer = DiskServer.id
        AND FileSystem.status IN (0, 1) -- PRODUCTION, DRAINING
        AND DiskServer.status IN (0, 1) -- PRODUCTION, DRAINING
      ORDER BY -- first prefer diskservers where no migrator runs and filesystems with no recalls
               DiskServer.nbMigratorStreams ASC, FileSystem.nbRecallerStreams ASC,
               -- then order by rate as defined by the function
               fileSystemRate(FileSystem.readRate, FileSystem.writeRate, FileSystem.nbReadStreams, FileSystem.nbWriteStreams,
                              FileSystem.nbReadWriteStreams, FileSystem.nbMigratorStreams, FileSystem.nbRecallerStreams) DESC,
               -- finally use randomness to avoid preferring always the same FS
               DBMS_Random.value) LOOP
    BEGIN
      -- lock the complete diskServer as we will update all filesystems
      SELECT id INTO unused FROM DiskServer WHERE id = f.DiskServerId FOR UPDATE NOWAIT;
      SELECT /*+ FIRST_ROWS(1) LEADING(D T StT C) */
             f.diskServerId, f.name, f.mountPoint, f.fileSystemId, D.path, D.id, D.castorfile, C.fileId, C.nsHost, C.fileSize, T.id, C.lastUpdateTime
        INTO diskServerId, diskServerName, mountPoint, fileSystemId, path, dci, castorFileId, fileId, nsHost, fileSize, tapeCopyId, lastUpdateTime
        FROM DiskCopy D, TapeCopy T, Stream2TapeCopy StT, Castorfile C
       WHERE decode(D.status, 10, D.status, NULL) = 10 -- CANBEMIGR
         AND D.filesystem = f.fileSystemId
         AND StT.parent = streamId
         AND T.status = 2 -- WAITINSTREAMS
         AND StT.child = T.id
         AND T.castorfile = D.castorfile
         AND C.id = D.castorfile
         AND ROWNUM < 2 FOR UPDATE OF T.id NOWAIT;
      -- found something on this filesystem, no need to go on
      diskServerId := f.DiskServerId;
      fileSystemId := f.fileSystemId;
      EXIT;
    EXCEPTION WHEN NO_DATA_FOUND OR LockError THEN
      -- either the filesystem is already locked or we found nothing,
      -- let's go to the next one
      NULL;
    END;
  END LOOP;

  IF tapeCopyId = 0 THEN
    -- Nothing found, return; locks will be released by the caller
    RETURN;
  END IF;
  
  UPDATE Stream
     SET lastFileSystemUsed = fileSystemId,
         lastButOneFileSystemUsed = lastFileSystemUsed,
         lastFileSystemChange = getTime()
   WHERE id = streamId;

  -- Update Filesystem state
  updateFSMigratorOpened(diskServerId, fileSystemId, 0);
END;
/

/* get a candidate for migration */

create or replace PROCEDURE tg_getFileToMigrate(transactionId IN NUMBER,
                                                ret OUT INTEGER,
                                                outVid OUT NOCOPY VARCHAR2,
                                                outputFile OUT castorTape.FileToMigrateCore_cur) AS
  CONSTRAINT_VIOLATED EXCEPTION;
  PRAGMA EXCEPTION_INIT(CONSTRAINT_VIOLATED, -1);
  strId NUMBER;
  policy VARCHAR2(100);
  ds VARCHAR2(2048);
  mp VARCHAR2(2048);
  path  VARCHAR2(2048);
  dcid NUMBER:=0;
  cfId NUMBER;
  fileId NUMBER;
  nsHost VARCHAR2(2048);
  fileSize  INTEGER;
  tcId  INTEGER:=0;
  lastUpdateTime NUMBER;
  knownName VARCHAR2(2048);
  newFseq NUMBER;
  trId NUMBER;
  srId NUMBER;
  unused INTEGER;
BEGIN
  ret:=0;
  BEGIN
    -- last fseq is the first fseq value available for this migration
    SELECT streammigration, lastfseq, id INTO strId, newFseq, trId
      FROM TapeGatewayRequest
     WHERE vdqmvolreqid = transactionId
     FOR UPDATE;

    SELECT vid INTO outVid
      FROM Tape
     WHERE id IN
         (SELECT tape
            FROM Stream
           WHERE id = strId);
  EXCEPTION WHEN NO_DATA_FOUND THEN
    ret:=-2;   -- stream is over
    RETURN;
  END;

  BEGIN  
    SELECT 1 INTO unused FROM dual 
      WHERE EXISTS (SELECT 'x' FROM Stream2TapeCopy 
                      WHERE parent=strId);
  EXCEPTION WHEN NO_DATA_FOUND THEN
    ret:=-1;   -- no more files
    RETURN;
  END;
 -- lock to avoid deadlock with mighunter 
 SELECT id INTO strId FROM stream WHERE id=strId FOR UPDATE;
  -- get the policy name and execute the policy
  BEGIN
    SELECT migrSelectPolicy INTO policy
      FROM Stream, TapePool
     WHERE Stream.id = strId
       AND Stream.tapePool = TapePool.id;
    -- check for NULL value
    IF policy IS NULL THEN
      policy := 'defaultMigrSelPolicy';
    END IF;
  EXCEPTION WHEN NO_DATA_FOUND THEN
    policy := 'defaultMigrSelPolicy';
  END;

  IF policy like 'defaultMigrSelPolicy' THEN
    -- default policy
    tg_defaultMigrSelPolicy(strId,ds,mp,path,dcid ,cfId, fileId,nsHost, fileSize,tcId, lastUpdateTime);
  ELSIF  policy LIKE 'repackMigrSelPolicy' THEN
    -- repack policy
    tg_repackMigrSelPolicy(strId,ds,mp,path,dcid ,cfId, fileId,nsHost, fileSize,tcId, lastUpdateTime);
  ELSIF  policy LIKE 'drainDiskMigrSelPolicy' THEN
    -- drain disk policy
    tg_drainDiskMigrSelPolicy(strId,ds,mp,path,dcid ,cfId, fileId,nsHost, fileSize,tcId, lastUpdateTime);
  ELSE
    -- default if we don't know at this point
    tg_defaultMigrSelPolicy(strId,ds,mp,path,dcid ,cfId, fileId,nsHost, fileSize,tcId, lastUpdateTime);
  END IF;

  IF tcId = 0 OR dcid=0 THEN
    ret := -1; -- the migration selection policy didn't find any candidate
    COMMIT;
    RETURN;
  END IF;

  -- Here we found a tapeCopy and we process it
  -- update status of selected tapecopy and stream
  UPDATE TapeCopy SET status = 3 -- SELECTED
   WHERE id = tcId;
  -- detach the tapecopy from the stream now that it is SELECTED;
  DELETE FROM Stream2TapeCopy
   WHERE child = tcId;


  SELECT lastknownfilename INTO knownName
    FROM CastorFile
   WHERE id = cfId; -- we rely on the check done before

  BEGIN
    INSERT INTO TapeGatewaySubRequest
      (fseq, id, tapecopy, request,diskcopy)
      VALUES (newFseq, ids_seq.nextval, tcId,trId, dcid)
      RETURNING id INTO srId;
      
    INSERT INTO id2type
      (id,type)
      VALUES (srId,180);
  

   UPDATE TapeGatewayRequest
     SET lastfseq=lastfseq+1
     WHERE id=trId; --increased when we have the proper value

   OPEN outputFile FOR
     SELECT fileId,nshost,lastUpdateTime,ds,mp,path,knownName,fseq,filesize,id
       FROM tapegatewaysubrequest
      WHERE id = srId;
      
  EXCEPTION WHEN CONSTRAINT_VIOLATED THEN
    -- detach from this stream
    -- resurrect the candidate if there is no other stream containing the tapecopy
    DELETE FROM stream2tapecopy WHERE child= tcid AND parent= strid;
    UPDATE tapecopy SET status=1 WHERE id = tcId AND NOT EXISTS (SELECT 'x' FROM stream2tapecopy WHERE child=tcId);
    -- recursion to avoid extra mount
    tg_getFileToMigrate(transactionId,ret,outVid,outputFile);
 END;
 COMMIT;
END;
/


/* get a candidate for recall */


create or replace
PROCEDURE tg_getFileToRecall (transactionId IN NUMBER, 
                                                ret OUT INTEGER, 
                                                vidOut OUT NOCOPY VARCHAR2,
                                                outputFile OUT castorTape.FileToRecallCore_Cur) AS
 trId INTEGER;
 ds VARCHAR2(2048);
 mp VARCHAR2(2048);
 path VARCHAR2(2048); 
 segId NUMBER;
 dcId NUMBER;
 tcId NUMBER;
 tapeId NUMBER;
 srId NUMBER;
 newFseq INTEGER;
 cfId NUMBER;

BEGIN 
  ret:=0;
  BEGIN
  -- master lock on the taperequeststate
    SELECT id, taperecall INTO trId, tapeid 
      FROM TapeGatewayRequest 
      WHERE vdqmVolReqId=transactionid 
      FOR UPDATE;
  
    SELECT vid INTO vidOut 
      FROM Tape 
      WHERE id=tapeId;

  EXCEPTION WHEN  NO_DATA_FOUND THEN
     ret:=-2; -- ERROR
     RETURN;
  END; 
  
  BEGIN
   
    SELECT id INTO segId FROM 
      ( SELECT id FROM Segment 
         WHERE  tape = tapeId  
         AND status=0 
         ORDER BY fseq ASC) 
      WHERE ROWNUM<2;

    SELECT fseq, copy INTO  newFseq,tcId 
      FROM Segment 
      WHERE id = segId;

    SELECT Castorfile.id INTO cfId 
      FROM Castorfile,TapeCopy 
      WHERE  Castorfile.id=TapeCopy.castorfile 
      AND TapeCopy.id=tcId 
      FOR UPDATE OF CastorFile.id;

  EXCEPTION WHEN NO_DATA_FOUND THEN
     ret:=-1; -- NO MORE FILES
     COMMIT;
     RETURN;
  END;

  DECLARE
    application_error EXCEPTION;
    PRAGMA EXCEPTION_INIT(application_error,-20115);
  BEGIN
    bestFileSystemForSegment(segId,ds,mp,path,dcId);
  EXCEPTION WHEN  application_error  OR NO_DATA_FOUND THEN 
    ret:=-3;
    COMMIT;
    RETURN;
  END;
 
  INSERT INTO TapeGatewaySubRequest
    (fseq, id, tapecopy, request,diskcopy) 
    VALUES (newFseq, ids_seq.nextval, tcId,trId, dcid) 
    RETURNING id INTO srId;

  INSERT INTO id2type 
    (id,type) 
    VALUES (srId,180);    
  
  UPDATE Segment SET status=7 
    WHERE id=segId 
    AND status = 0;

  OPEN outputFile FOR 
    SELECT castorfile.fileid, castorfile.nshost, ds, mp, path, newFseq, srId 
      FROM TapeCopy,Castorfile  
      WHERE tcId=TapeCopy.id 
      AND CastorFile.id=TapeCopy.castorfile;
END;
/


/* get the information from the db for a successful migration */


create or replace
PROCEDURE tg_getRepackVidAndFileInfo( inputFileId IN NUMBER,
                                      inputNsHost IN VARCHAR2,
                                      inFseq IN INTEGER,
                                      transactionId IN NUMBER, 
                                      byteTransfered IN NUMBER,
                                      outRepackVid OUT NOCOPY VARCHAR2,
                                      strVid OUT NOCOPY VARCHAR,
                                      outCopyNb OUT INTEGER,
                                      outLastTime OUT NUMBER,
                                      ret OUT INTEGER) AS 
  cfId NUMBER;
  fs NUMBER;
BEGIN
  outrepackvid:=NULL;
   -- ignore the repack state
  SELECT  castorfile.lastupdatetime, castorfile.id, fileSize INTO outlasttime, cfId, fs
    FROM CastorFile 
    WHERE castorfile.fileid = inputFileId 
    AND castorfile.nshost = inputNsHost;
  IF fs <> byteTransfered THEN
  -- fail the file
    tg_failfiletransfer(transactionid,inputfileid, inputNsHost, inFseq,  1613); -- wrongfilesize
    COMMIT;
    ret:=-1;
    RETURN;
  ELSE
    ret:=0;
  END IF;
      
  SELECT copynb INTO outcopynb 
    FROM TapeCopy,TapeGatewaySubRequest, TapeGatewayRequest  
    WHERE TapeCopy.id = TapeGatewaySubRequest.tapecopy 
    AND TapeGatewayRequest.id = TapeGatewaySubRequest.request
    AND Tapecopy.castorfile = cfId
    AND TapeGatewaySubRequest.fseq= inFseq
    AND TapeGatewayRequest.vdqmVolReqId = transactionId;
 
  SELECT vid INTO strVid 
    FROM Tape,Stream,TapeGatewayRequest 
    WHERE  Tape.id = Stream.tape
    AND  TapeGatewayRequest.streammigration = stream.id
    AND  TapeGatewayRequest.vdqmVolReqId = transactionId;

  BEGIN 
   --REPACK case
    -- Get the repackvid field from the existing request (if none, then we are not in a repack process
     SELECT StageRepackRequest.repackvid INTO outRepackVid
       FROM SubRequest, StageRepackRequest
       WHERE StageRepackRequest.id = SubRequest.request
       AND SubRequest.status = 12 -- SUBREQUEST_REPACK
       AND SubRequest.castorFile = cfId
       AND ROWNUM < 2;
  EXCEPTION WHEN NO_DATA_FOUND THEN
   -- standard migration
    null;
  END;
END;
/



/* get the information from the db for a successful recall */

CREATE OR REPLACE PROCEDURE tg_getSegmentInfo ( transactionId IN NUMBER,
                                                inFileId IN NUMBER, 
                                                inHost IN VARCHAR2,
                                                inFseq IN INTEGER, 
                                                outVid OUT NOCOPY VARCHAR2, 
                                                outCopy OUT INTEGER ) AS
 trId NUMBER;
 cfId NUMBER;

BEGIN
  SELECT Tape.vid, TapeGatewayRequest.id  INTO outVid, trId 
    FROM Tape,TapeGatewayRequest
    WHERE Tape.id=TapeGatewayRequest.taperecall 
    AND TapeGatewayRequest.vdqmvolreqid= transactionId
    AND Tape.tpmode = 0;

  SELECT copynb INTO outCopy FROM TapeCopy, TapeGatewaySubRequest
    WHERE TapeCopy.id = TapeGatewaySubRequest.tapecopy
    AND TapeGatewaySubRequest.fseq = inFseq
    AND TapeGatewaySubRequest.request = trId;
END;
/

/* get the stream without any tape associated */

CREATE OR REPLACE PROCEDURE tg_getStreamsWithoutTapes (strList OUT castorTape.Stream_Cur) AS

BEGIN
 -- get request in status TO_BE_RESOLVED
 OPEN strList FOR
    SELECT Stream.id, Stream.initialsizetotransfer, Stream.status, Stream.tapepool, TapePool.name
      FROM Stream,Tapepool 
      WHERE Stream.id IN 
        ( SELECT streamMigration 
            FROM TapeGatewayRequest 
            WHERE status=0 ) 
      AND Stream.tapepool=TapePool.id 
      FOR UPDATE OF Stream.id SKIP LOCKED;   
END;
/

/* get tape with a pending request in VDQM */


CREATE OR REPLACE PROCEDURE tg_getTapesWithDriveReqs (timeLimit IN NUMBER,
                                                      taperequest OUT castorTape.tapegatewayrequest_Cur) AS

 trIds "numList";

BEGIN 
-- get requests	 in  WAITING_TAPESERVER and ONGOING

    SELECT id BULK COLLECT into trIds 
      FROM TapeGatewayRequest 
      WHERE status IN (2,3) 
      AND  gettime() - TapeGatewayRequest.lastVdqmPingTime > timeLimit 
      FOR UPDATE SKIP LOCKED; 
	
  FORALL i IN trids.FIRST .. trids.LAST
      UPDATE TapeGatewayRequest 
        SET lastVdqmPingTime = gettime() 
        WHERE id = trIds(i);
 
  OPEN taperequest FOR
      SELECT TapeGatewayRequest.accessMode, TapeGatewayRequest.id,TapeGatewayRequest.starttime,
             TapeGatewayRequest.lastvdqmpingtime, TapeGatewayRequest.vdqmvolreqid,
             TapeGatewayRequest.status, Tape.vid  
         FROM Stream,TapeGatewayRequest,Tape 
         WHERE TapeGatewayRequest.id  IN 
            (SELECT /*+ CARDINALITY(trTable 5) */ * 
               FROM TABLE(trIds) trTable)  
         AND  Stream.id = TapeGatewayRequest.streamMigration 
         AND Stream.tape=Tape.id 
         AND TapeGatewayRequest.accessMode=1

       UNION ALL

      SELECT TapeGatewayRequest.accessMode, TapeGatewayRequest.id, TapeGatewayRequest.starttime, 
             TapeGatewayRequest.lastvdqmpingtime, TapeGatewayRequest.vdqmvolreqid,
             TapeGatewayRequest.status, Tape.vid  
        FROM tapegatewayrequest,Tape 
        WHERE Tape.id=TapeGatewayRequest.tapeRecall 
        AND TapeGatewayRequest.accessMode=0 
        AND TapeGatewayRequest.id IN 
            (SELECT /*+ CARDINALITY(trTable 5) */ * 
             FROM TABLE(trIds) trTable);
END;
/

/* get a tape without any drive requests sent to VDQM */



create or replace
PROCEDURE tg_getTapeWithoutDriveReq ( reqId OUT NUMBER,tapeMode OUT NUMBER, tapeSide OUT INTEGER, tapeVid OUT NOCOPY VARCHAR2) AS
BEGIN
-- get a request in TO_BE_SENT_TO_VDQM
   SELECT id, accessmode INTO reqId, tapeMode 
     FROM TapeGatewayRequest 
     WHERE status=1 AND ROWNUM < 2 FOR UPDATE SKIP LOCKED;
  IF tapemode = 0 THEN
    -- read
    SELECT  tpmode, side, vid INTO tapeMode, tapeSide, tapeVid
       FROM TapeGatewayRequest,Tape 
       WHERE TapeGatewayRequest.id = reqId
       AND Tape.id=TapeGatewayRequest.tapeRecall;
  ELSE
     -- write  
    SELECT  tpmode, side, vid INTO tapeMode, tapeSide, tapeVid
       FROM TapeGatewayRequest,Tape,Stream 
       WHERE TapeGatewayRequest.id = reqId
       AND Stream.id = TapeGatewayRequest.streamMigration 
       AND Stream.tape=Tape.id;
  END IF;
EXCEPTION WHEN NO_DATA_FOUND THEN
       reqId:=0; 
END;
/


/* get tape to release in VMGR */

create or replace
PROCEDURE  tg_getTapeToRelease ( inputVdqmReqId IN INTEGER, 
                                                   outputTape OUT NOCOPY VARCHAR2, 
                                                   outputMode OUT INTEGER ) AS
  strId NUMBER;
  tpId NUMBER;
BEGIN
  SELECT  accessmode, streammigration, tapeRecall 
     INTO  outputmode, strId, tpId 
    FROM TapeGatewayRequest
    WHERE vdqmvolreqid = inputvdqmreqid;
    
   IF outputmode = 0 THEN
     -- read case
     SELECT vid INTO outputTape 
       FROM Tape
       WHERE id=tpId; 
   ELSE
     -- write case
     SELECT Tape.vid INTO outputtape 
       FROM Tape,Stream 
       WHERE Stream.id=strId AND Stream.tape=Tape.id;
   END IF;
EXCEPTION WHEN NO_DATA_FOUND THEN
 -- already cleaned by the checker
 null;
END;
/


/* invalidate a file that it is not possible to tape as candidate to migrate or recall */

CREATE OR REPLACE PROCEDURE tg_invalidateFile (transactionId IN NUMBER,
                                               inputFileId IN NUMBER, 
                                               inputNsHost IN VARCHAR2,
                                               inputFseq IN INTEGER,
                                               inputErrorCode IN INTEGER) AS
BEGIN

  UPDATE TapeGatewayRequest SET lastfseq=lastfseq-1 
    WHERE vdqmvolreqid=transactionid; 
  
  tg_failfiletransfer(transactionid,inputfileid, inputNsHost, inputFseq, inputErrorCode);

END;
/

/* restart taperequest which had problems */




create or replace
PROCEDURE tg_restartLostReqs(trIds IN castor."cnumList") AS

 vdqmId INTEGER;

BEGIN

 FOR  i IN trIds.FIRST .. trIds.LAST LOOP   
    BEGIN
 
     --  STATUS ONGOING: these are the ones which were ongoing ... crash clean up needed 
      SELECT vdqmvolreqid INTO vdqmId 
        FROM  TapeGatewayRequest 
        WHERE id=trIds(i) ;
      tg_endTapeSession(vdqmId,0);

    EXCEPTION WHEN NO_DATA_FOUND THEN
     NULL;
    END;
 END LOOP;
 COMMIT;
END;
/


/* update the db after a successful migration */

create or replace PROCEDURE tg_setFileMigrated  (transactionId IN NUMBER, 
                                                inputFileId  IN NUMBER,
                                                inputNsHost IN VARCHAR2, 
                                                inputFseq IN INTEGER, 
                                                inputFileTransaction NUMBER,
                                                streamReport OUT castor.StreamReport_Cur) AS
 trId NUMBER;
 tcNumb INTEGER;
 cfId NUMBER;
 tcId NUMBER;
 srId NUMBER;
 tcIds "numList";
 srIds "numList";
 dcId NUMBER;

BEGIN
 
  SELECT id  INTO trId FROM TapeGatewayRequest 
    WHERE TapeGatewayRequest.vdqmvolreqid=transactionid 
    FOR UPDATE;
  
  SELECT id  INTO cfId FROM CastorFile 
    WHERE  fileid=inputFileId 
    AND nshost=inputNsHost 
    FOR UPDATE;
   
  srId:=NULL; 
  DELETE FROM TapeGatewaySubRequest 
    WHERE id = inputFileTransaction 
    AND fseq = inputfseq
    RETURNING tapecopy, id, diskcopy INTO tcId, srId, dcId;
    
  IF srid IS NULL THEN
    raise NO_DATA_FOUND;
  END IF;

  DELETE FROM id2type WHERE id= srId;

  UPDATE tapecopy SET status=5 WHERE id=tcId;
   
  SELECT count(*) INTO tcNumb 
    FROM tapecopy 
    WHERE castorfile = cfId  
    AND STATUS != 5;

  -- let's check if another copy should be done
  IF tcNumb = 0 THEN
     UPDATE DiskCopy SET status=0 
       WHERE castorfile = cfId
       AND status=10;

     DELETE FROM tapecopy 
       WHERE castorfile=cfId 
       RETURNING id BULK COLLECT INTO tcIds;
   
     FORALL i IN tcIds.FIRST .. tcIds.LAST
       DELETE FROM id2type 
         WHERE id=tcIds(i);
  END IF;

  -- archive Repack requests should any be in the db
  FOR i IN (SELECT id FROM SubRequest WHERE castorfile=cfId AND status=12) LOOP
    archivesubreq(i.id, 8);
  END LOOP;
  
  -- return data for informing the rmMaster
  OPEN streamReport FOR
   SELECT DiskServer.name,FileSystem.mountpoint 
     FROM DiskServer,FileSystem,DiskCopy 
     WHERE DiskCopy.id=dcId 
     AND DiskCopy.filesystem=FileSystem.id 
     AND FileSystem.diskserver = DiskServer.id;
  COMMIT;
END;
/

/* repack start the migration phase */

create or replace PROCEDURE tg_startRepackMigration
(srId IN INTEGER, cfId IN INTEGER, dcId IN INTEGER,
 reuid OUT INTEGER, regid OUT INTEGER) AS
  fs NUMBER;
  svcClassId NUMBER;
BEGIN
  UPDATE DiskCopy SET status = 6  -- DISKCOPY_STAGEOUT
   WHERE id = dcId RETURNING diskCopySize INTO fs;
  -- update the Repack subRequest for this file. The status REPACK
  -- stays until the migration to the new tape is over.
  UPDATE SubRequest
     SET diskCopy = dcId, status = 12  -- REPACK
   WHERE id = srId;   
  -- get the service class, uid and gid
  SELECT R.svcClass, euid, egid INTO svcClassId, reuid, regid
    FROM StageRepackRequest R, SubRequest
   WHERE SubRequest.request = R.id
     AND SubRequest.id = srId;
  -- create the required number of tapecopies for the files
  -- one tapecopy by definition because we cannot repack the same file more than once at the same time.
  internalPutDoneFunc(cfId, fs, 0, 1, svcClassId);
  -- set svcClass in the CastorFile for the migration
  UPDATE CastorFile SET svcClass = svcClassId WHERE id = cfId;
  -- update remaining STAGED diskcopies to CANBEMIGR too
  -- we may have them as result of disk2disk copies, and so far
  -- we only dealt with dcId
  UPDATE DiskCopy SET status = 10  -- DISKCOPY_CANBEMIGR
   WHERE castorFile = cfId AND status = 0;  -- DISKCOPY_STAGED
END;
/



/* update the db after a successful recall */

create or replace PROCEDURE tg_setFileRecalled (transactionId IN NUMBER, 
                                                inputFileId  IN NUMBER,
                                                inputNsHost IN VARCHAR2, 
                                                inputFseq IN NUMBER, 
                                                inputFileTransaction IN NUMBER,
                                                streamReport OUT castor.StreamReport_Cur) AS
  tcId NUMBER;
  dcId NUMBER;
  cfId NUMBER;
  unused NUMBER;
  segId NUMBER;
  tpId NUMBER;
  srId NUMBER;
  subRequestId NUMBER;
  requestId NUMBER;
  reqType NUMBER;
  fs NUMBER;
  gcw NUMBER;
  gcwProc VARCHAR(2048);
  ouid INTEGER;
  ogid INTEGER;
  svcClassId NUMBER;
  trId NUMBER;
  missingTCs INTEGER;

BEGIN
  -- take locks
  SELECT id, taperecall INTO trId, tpId 
    FROM TapeGatewayRequest 
    WHERE TapeGatewayRequest.vdqmvolreqid=transactionid 
    FOR UPDATE;

  SELECT id, fileSize into cfId, fs FROM CastorFile 
    WHERE  fileid=inputFileId 
    AND nshost=inputNsHost 
    FOR UPDATE;
   
  srid:=NULL; 
  DELETE FROM TapeGatewaySubRequest 
    WHERE id= inputFileTransaction
    AND fseq = inputFseq 
    RETURNING tapecopy,id, diskcopy INTO tcId, srId, dcId;
    
  IF srid IS NULL THEN
    raise NO_DATA_FOUND;
  END IF; 
    
  DELETE FROM id2type WHERE id = srId;
 
  -- missing tapecopies handling
  SELECT missingCopies INTO missingTCs
    FROM TapeCopy WHERE id = tcId;
    
  -- delete tapecopies
  deleteTapeCopies(cfId);
  
  -- update diskcopy status, size and gweight
  SELECT SubRequest.id, SubRequest.request, Id2Type.type 
    INTO subRequestId, requestId, reqType
    FROM SubRequest, Id2Type
   WHERE SubRequest.request = Id2Type.id
     AND SubRequest.diskcopy = dcId;
  SELECT Request.svcClass, euid, egid INTO svcClassId, ouid, ogid
    FROM (SELECT id, svcClass, euid, egid FROM StageGetRequest UNION ALL
          SELECT id, svcClass, euid, egid FROM StagePrepareToGetRequest UNION ALL
          SELECT id, svcClass, euid, egid FROM StageUpdateRequest UNION ALL
          SELECT id, svcClass, euid, egid FROM StagePrepareToUpdateRequest UNION ALL
          SELECT id, svcClass, euid, egid FROM StageRepackRequest) Request
    WHERE Request.id = requestId;
    gcwProc := castorGC.getRecallWeight(svcClassId);
  EXECUTE IMMEDIATE 'BEGIN :newGcw := ' || gcwProc || '(:size); END;'
    USING OUT gcw, IN fs;
  UPDATE DiskCopy 
    SET status = decode(missingTCs, 0, 0, 10), -- DISKCOPY_STAGED if missingTCs = 0, otherwise CANBEMIGR
        lastAccessTime = getTime(),  -- for the GC, effective lifetime of this diskcopy starts now
        gcWeight = gcw,
        diskCopySize = fs
    WHERE id = dcid;
  
  IF reqType = 119 THEN  -- OBJ_StageRepackRequest
    tg_startRepackMigration(subRequestId, cfId, dcid, ouid, ogid);
  ELSE
    -- restart this subrequest if it's not a repack one
    UPDATE SubRequest
       SET status = 1, getNextStatus = 1,  -- SUBREQUEST_RESTART, GETNEXTSTATUS_FILESTAGED
           lastModificationTime = getTime(), parent = 0
       WHERE id = subRequestId;
    -- and trigger new migrations if missing tape copies were detected
    IF missingTCs > 0 THEN
      DECLARE
        newTcId INTEGER;
      BEGIN
        FOR i IN 1..missingTCs LOOP
          INSERT INTO TapeCopy (id, copyNb, castorFile, status, nbRetry, missingCopies)
          VALUES (ids_seq.nextval, 0, cfId, 0, 0, 0)  -- TAPECOPY_CREATED
          RETURNING id INTO newTcId;
          INSERT INTO Id2Type (id, type) VALUES (newTcId, 30); -- OBJ_TapeCopy
        END LOOP;
      END;
    END IF;
  END IF;
  
  -- restart other requests waiting on this recall
  UPDATE SubRequest
     SET status = 1, getNextStatus = 1,  -- SUBREQUEST_RESTART, GETNEXTSTATUS_FILESTAGED
         lastModificationTime = getTime(), parent = 0
     WHERE parent = subRequestId;

  -- trigger the creation of additional copies of the file, if necessary.
  replicateOnClose(cfId, ouid, ogid);

  -- return data for informing the rmMaster
  OPEN streamReport FOR
    SELECT DiskServer.name, FileSystem.mountpoint 
      FROM DiskServer, FileSystem, DiskCopy
      WHERE DiskCopy.id = dcId
      AND DiskCopy.filesystem = FileSystem.id 
      AND FileSystem.diskserver = DiskServer.id;
  COMMIT;
END;
/


/* save in the db the results returned by the retry policy for migration */

CREATE OR REPLACE PROCEDURE  tg_setMigRetryResult( tcToRetry IN castor."cnumList", 
                                                   tcToFail IN castor."cnumList"  ) AS
 srId NUMBER;
 cfId NUMBER;

BEGIN
   -- check because oracle cannot handle empty buffer
  IF tcToRetry( tcToRetry.FIRST) != -1 THEN
 
    -- restarted the one to be retried
    FORALL i IN tctoretry.FIRST .. tctoretry.LAST 
      UPDATE TapeCopy SET status=1,nbretry = nbretry+1  
        WHERE id = tcToRetry(i);    

  END IF;
  
  -- check because oracle cannot handle empty buffer
  IF tcToFail(tcToFail.FIRST) != -1 THEN 
    -- fail the tapecopies   
    FORALL i IN tctofail.FIRST .. tctofail.LAST 
      UPDATE TapeCopy SET status=6 
        WHERE id = tcToFail(i);

    -- fail repack subrequests
    FOR i IN tcToFail.FIRST .. tcToFail.LAST LOOP
        BEGIN 
        -- we don't need a lock on castorfile because we cannot have a parallel migration of the same file using repack
          SELECT SubRequest.id, SubRequest.castorfile into srId, cfId 
            FROM SubRequest,TapeCopy 
            WHERE TapeCopy.id = tcToFail(i) 
            AND SubRequest.castorfile = TapeCopy.castorfile 
            AND subrequest.status = 12;

          -- STAGED because the copy on disk most probably is valid and the failure of repack happened during the migration 

          UPDATE DiskCopy SET status=0 
            WHERE castorfile = cfId 
            AND status=10; -- otherwise repack will wait forever

          archivesubreq(srId,9);

        EXCEPTION WHEN NO_DATA_FOUND THEN
          NULL;
        END;
     END LOOP;
  END IF;
  
  COMMIT;
END;
/

/* save in the db the results returned by the retry policy for recall */

CREATE OR REPLACE PROCEDURE  tg_setRecRetryResult( tcToRetry IN castor."cnumList", 
                                                   tcToFail IN castor."cnumList"  ) AS
 tapeId NUMBER;
 cfId NUMBER;

BEGIN
  -- I restart the recall that I want to retry
  -- check because oracle cannot handle empty buffer
  IF tcToRetry(tcToRetry.FIRST) != -1 THEN 

    -- tapecopy => TOBERECALLED
    FORALL i IN tcToRetry.FIRST .. tcToRetry.LAST
      UPDATE TapeCopy SET status=4, errorcode=0, nbretry= nbretry+1 
        WHERE id=tcToRetry(i);
    
    -- segment => UNPROCESSED
    -- tape => PENDING if UNUSED OR FAILED with still segments unprocessed
    FOR i IN tcToRetry.FIRST .. tcToRetry.LAST LOOP
      UPDATE Segment SET status=0
       WHERE copy = tcToRetry(i)
      RETURNING tape INTO tapeId;
      UPDATE Tape set status = 8
       WHERE id = tapeId AND status IN (0, 6);
    END LOOP;
  END IF;
  
  -- I mark as failed the hopeless recall
  -- check because oracle cannot handle empty buffer
  IF tcToFail(tcToFail.FIRST) != -1 THEN
    FOR i IN tcToFail.FIRST .. tcToFail.LAST  LOOP

      -- lock castorFile	
      SELECT castorFile INTO cfId 
        FROM TapeCopy,CastorFile
        WHERE TapeCopy.id = tcToFail(i) 
        AND CastorFile.id = TapeCopy.castorfile 
        FOR UPDATE OF castorfile.id;

      -- fail diskcopy
      UPDATE DiskCopy SET status = 4 -- DISKCOPY_FAILED
        WHERE castorFile = cfId 
        AND status = 2; -- DISKCOPY_WAITTAPERECALL   
      
      -- Drop tape copies. Ideally, we should keep some track that
      -- the recall failed in order to prevent future recalls until some
      -- sort of manual intervention. For the time being, as we can't
      -- say whether the failure is fatal or not, we drop everything
      -- and we won't deny a future request for recall.
      deleteTapeCopies(cfId);
      
      -- fail subrequests
      UPDATE SubRequest 
        SET status = 7, -- SUBREQUEST_FAILED
            getNextStatus = 1, -- GETNEXTSTATUS_FILESTAGED (not strictly correct but the request is over anyway)
            lastModificationTime = getTime(),
            errorCode = 1015,  -- SEINTERNAL
            errorMessage = 'File recall from tape has failed, please try again later',
            parent = 0
        WHERE castorFile = cfId 
        AND status IN (4, 5); -- WAITTAPERECALL, WAITSUBREQ
    
    END LOOP;
  END IF;
  COMMIT;
END;
/


/* update the db when a tape session is started */

create or replace
PROCEDURE  tg_startTapeSession ( inputVdqmReqId IN NUMBER, 
                                 outVid OUT NOCOPY VARCHAR2, 
                                 outMode OUT INTEGER, 
                                 ret OUT INTEGER, 
                                 outDensity OUT NOCOPY VARCHAR2, 
                                 outLabel OUT NOCOPY VARCHAR2 ) AS
 reqId NUMBER;
 tpId NUMBER;
 segToDo INTEGER;
 tcToDo INTEGER;
 strId NUMBER;
 unused NUMBER;

BEGIN

  ret:=-2;
  -- set the request to ONGOING
  UPDATE TapeGatewayRequest SET status=3 
    WHERE vdqmVolreqid = inputVdqmReqId  
    AND status =2 
    RETURNING id, accessmode, streammigration, taperecall INTO reqId, outMode,strId, tpId;
  
  IF reqId IS NULL THEN
  
   ret:=-2; -- UNKNOWN request
 
  ELSE  
 
    IF outMode = 0 THEN
      -- read tape mounted
      
       BEGIN  
        SELECT 1 INTO unused FROM dual 
         WHERE EXISTS (SELECT 'x' FROM Segment 
                          WHERE tape = tpId);
      EXCEPTION WHEN NO_DATA_FOUND THEN
        UPDATE TapeGatewayRequest SET lastvdqmpingtime=0 
          WHERE id=reqId; -- to force the cleanup
        ret:=-1; --NO MORE FILES
        COMMIT;
        RETURN;

      END;

      UPDATE Tape SET status=4 
          WHERE id = tpId
          AND tpmode=0  
          RETURNING vid, label, density INTO outVid, outLabel, outDensity; -- tape is MOUNTED 
        
      ret:=0;
    ELSE 
      -- write
      BEGIN  
        SELECT 1 INTO unused FROM dual 
         WHERE EXISTS (SELECT 'x' FROM Stream2TapeCopy 
                          WHERE parent=strId);
      EXCEPTION WHEN NO_DATA_FOUND THEN
        -- no more files
        UPDATE TapeGatewayRequest SET lastvdqmpingtime=0 
          WHERE id=reqId; -- to force the cleanup

        ret:=-1; --NO MORE FILES
        outVid:=NULL;
        COMMIT;
        RETURN;
      END;
      
      UPDATE Stream SET status=3 
       WHERE id = 
         ( SELECT streammigration 
             FROM TapeGatewayRequest 
             WHERE id=reqId ) 
             RETURNING tape INTO tpId; -- RUNNING

      UPDATE Tape SET status=4  
        WHERE id = tpId 
        RETURNING vid, label, density INTO outVid, outLabel, outDensity; -- MOUNTED

     ret:=0;

    END IF; 

  END IF;
  COMMIT;
END;
/

/* Check configuration */

create or replace
PROCEDURE tg_checkConfiguration AS
unused VARCHAR2(2048);
BEGIN
 SELECT value INTO unused FROM castorconfig WHERE class='tape' AND key='interfaceDaemon' AND value='tapegatewayd';
END;
/


/* delete streams for not existing tapepools */

create or replace
PROCEDURE tg_deleteStream( strId IN NUMBER ) AS
unused NUMBER;
tcIds "numList";
trId NUMBER;
BEGIN
 DELETE FROM tapegatewayrequest WHERE streammigration=strId RETURNING id INTO trId;
 DELETE FROM id2type WHERE id=trId;
 SELECT id INTO unused FROM Stream WHERE id=strId FOR UPDATE;
 DELETE FROM stream2tapecopy WHERE parent=strId RETURNING child BULK COLLECT INTO tcIds;
 FORALL i IN tcIds.FIRST .. tcIds.LAST
    UPDATE tapecopy SET status = 1 WHERE tcIds(i)=id AND NOT EXISTS (SELECT 'x' FROM stream2tapecopy WHERE stream2tapecopy.child=tcIds(i));
 DELETE FROM id2type where id=strId;
 DELETE FROM stream where id=strId;
END;
/
	

/* delete taperequest  for not existing tape */

create or replace
PROCEDURE tg_deleteTapeRequest( reqId IN NUMBER ) AS
  CONSTRAINT_VIOLATED EXCEPTION;
  PRAGMA EXCEPTION_INIT(CONSTRAINT_VIOLATED, -02292);
  tpId NUMBER;
  strId NUMBER;
  reqMode INTEGER;
  segNum INTEGER;
  cfId NUMBER;
  tcIds "numList";
  srIds "numList";
 
BEGIN

   --delete the tape request. subrequests don't exist
    DELETE FROM TapeGatewayRequest 
      WHERE id=reqId AND status=1  
      RETURNING taperecall, streammigration,accessmode
      INTO tpId, strid, reqMode;

    DELETE FROM id2type WHERE id= reqid;
    IF reqmode = 0 THEN
      SELECT id INTO tpId
        FROM Tape
        WHERE id =tpId
        FOR UPDATE;
        
      SELECT copy BULK COLLECT INTO tcIds FROM Segment WHERE tape=tpId;
    
      FOR i IN tcIds.FIRST .. tcIds.LAST  LOOP
        -- lock castorFile	
        SELECT castorFile INTO cfId 
          FROM TapeCopy,CastorFile
          WHERE TapeCopy.id = tcIds(i) 
          AND CastorFile.id = TapeCopy.castorfile 
          FOR UPDATE OF castorfile.id;

        -- fail diskcopy
        UPDATE DiskCopy SET status = 4 -- DISKCOPY_FAILED
          WHERE castorFile = cfId 
          AND status = 2; -- DISKCOPY_WAITTAPERECALL   

        deleteTapeCopies(cfId);
        UPDATE SubRequest 
          SET status = 7, -- SUBREQUEST_FAILED
              getNextStatus = 1, -- GETNEXTSTATUS_FILESTAGED (not strictly correct but the request is over anyway)
              lastModificationTime = getTime(),
              errorCode = 1015,  -- SEINTERNAL
              errorMessage = 'File recall from tape has failed, please try again later',
              parent = 0
          WHERE castorFile = cfId 
          AND status IN (4, 5); -- WAITTAPERECALL, WAITSUBREQ
    
      END LOOP;
    END IF;
    IF reqmode = 1 THEN
    -- write
    -- reset stream
       BEGIN
         DELETE FROM Stream WHERE id = strId;
         DELETE FROM id2type WHERE id=strId;

       EXCEPTION WHEN CONSTRAINT_VIOLATED THEN
       -- constraint violation and we cannot delete the stream because there is an entry in Stream2TapeCopy
         UPDATE Stream SET status = 6, tape = null, lastFileSystemChange = null
           WHERE id = strId;
       END;
       -- reset tape
       UPDATE Tape SET status=0, stream = null WHERE stream = strId;
  END IF;
END;
/

/* Recompile all invalid procedures, triggers and functions */
/************************************************************/
BEGIN
  FOR a IN (SELECT object_name, object_type
              FROM user_objects
             WHERE object_type IN ('PROCEDURE', 'TRIGGER', 'FUNCTION')
               AND status = 'INVALID')
  LOOP
    IF a.object_type = 'PROCEDURE' THEN
      EXECUTE IMMEDIATE 'ALTER PROCEDURE '||a.object_name||' COMPILE';
    ELSIF a.object_type = 'TRIGGER' THEN
      EXECUTE IMMEDIATE 'ALTER TRIGGER '||a.object_name||' COMPILE';
    ELSE
      EXECUTE IMMEDIATE 'ALTER FUNCTION '||a.object_name||' COMPILE';
    END IF;
  END LOOP;
END;
/

/* Flag the schema upgrade as COMPLETE */
/***************************************/
UPDATE UpgradeLog SET endDate = sysdate, state = 'COMPLETE'
 WHERE release = '2_1_9_5';
COMMIT;
