/******************************************************************************
 *              2.1.9-5_to_2.1.9-5-1.sql
 *
 * This file is part of the Castor project.
 * See http://castor.web.cern.ch/castor
 *
 * Copyright (C) 2003  CERN
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * @(#)$RCSfile: template.sql,v $ $Release: 1.2 $ $Release$ $Date: 2009/02/16 09:13:00 $ $Author: waldron $
 *
 * This script upgrades a CASTOR v2.1.9-5 DBNAME database into v2.1.9-5-1
 *
 * Fixes:
 * - #65961: stager_rm of ongoing transfers results in blocked job slots and
 *           database CPU load
 *
 * @author Castor Dev team, castor-dev@cern.ch
 *****************************************************************************/

/* Stop on errors */
WHENEVER SQLERROR EXIT FAILURE
BEGIN
  UPDATE UpgradeLog
     SET failureCount = failureCount + 1
   WHERE schemaVersion = '2_1_9_5'
     AND release = '2_1_9_5_1'
     AND state != 'COMPLETE';
  COMMIT;
END;
/

/* Version cross check and update */
DECLARE
  unused VARCHAR(100);
BEGIN
  SELECT release INTO unused FROM CastorVersion
   WHERE release = '2_1_9_5';
EXCEPTION WHEN NO_DATA_FOUND THEN
  -- Error, we can't apply this script
  raise_application_error(-20000, 'PL/SQL release mismatch. Please run previous upgrade scripts before this one.');
END;
/

INSERT INTO UpgradeLog (schemaVersion, release, type)
VALUES ('2_1_9_4', '2_1_9_5_1', 'TRANSPARENT');
COMMIT;


/* Update and revalidation of PL-SQL code */
/******************************************/

/* #65961: stager_rm of ongoing transfers results in blocked job slots and
 *           database CPU load
 */

/* PL/SQL method to archive a SubRequest */
CREATE OR REPLACE PROCEDURE archiveSubReq(srId IN INTEGER, finalStatus IN INTEGER) AS
  unused INTEGER;
  rId INTEGER;
  rname VARCHAR2(100);
  srIds "numList";
  clientId INTEGER;
BEGIN
  SELECT request INTO rId
    FROM SubRequest
   WHERE id = srId;
  BEGIN
    -- Lock the access to the Request
    SELECT Id2Type.id INTO rId
      FROM Id2Type
     WHERE id = rId FOR UPDATE;
    UPDATE SubRequest
       SET parent = NULL, diskCopy = NULL,  -- unlink this subrequest as it's dead now
           lastModificationTime = getTime(),
           status = finalStatus
     WHERE id = srId;
    BEGIN
      -- Try to see whether another subrequest in the same
      -- request is still being processed
      SELECT id INTO unused FROM SubRequest
       WHERE request = rId AND status NOT IN (8, 9) AND ROWNUM < 2;  -- all but {FAILED_,}FINISHED
    EXCEPTION WHEN NO_DATA_FOUND THEN
      -- All subrequests have finished, we can archive
      SELECT object INTO rname
        FROM Id2Type, Type2Obj
       WHERE id = rId
         AND Type2Obj.type = Id2Type.type;
      -- drop the associated Client entity and all Id2Type entries
      EXECUTE IMMEDIATE
        'BEGIN SELECT client INTO :cId FROM '|| rname ||' WHERE id = :rId; END;'
        USING OUT clientId, IN rId;
      DELETE FROM Client WHERE id = clientId;
      DELETE FROM Id2Type WHERE id IN (rId, clientId);
      SELECT id BULK COLLECT INTO srIds
        FROM SubRequest
       WHERE request = rId;
      FORALL i IN srIds.FIRST .. srIds.LAST
        DELETE FROM Id2Type WHERE id = srIds(i);
      -- archive the successful subrequests      
      UPDATE /*+ INDEX(SubRequest I_SubRequest_Request) */ SubRequest
         SET status = 11    -- ARCHIVED
       WHERE request = rId
         AND status = 8;  -- FINISHED
    END;
  EXCEPTION WHEN NO_DATA_FOUND THEN
    -- No data found here means that the Id2Type entry is not there
    -- and the subrequest was already archived: just update this subrequest,
    -- don't bother with the whole request. Note that this could
    -- happen only if someone archives an already archived subrequest.
    UPDATE SubRequest
       SET lastModificationTime = getTime(),
           status = finalStatus
     WHERE id = srId;
  END;
END;
/


/* Recompile all invalid procedures, triggers and functions */
/************************************************************/
BEGIN
  FOR a IN (SELECT object_name, object_type
              FROM user_objects
             WHERE object_type IN ('PROCEDURE', 'TRIGGER', 'FUNCTION')
               AND status = 'INVALID')
  LOOP
    IF a.object_type = 'PROCEDURE' THEN
      EXECUTE IMMEDIATE 'ALTER PROCEDURE '||a.object_name||' COMPILE';
    ELSIF a.object_type = 'TRIGGER' THEN
      EXECUTE IMMEDIATE 'ALTER TRIGGER '||a.object_name||' COMPILE';
    ELSE
      EXECUTE IMMEDIATE 'ALTER FUNCTION '||a.object_name||' COMPILE';
    END IF;
  END LOOP;
END;
/

/* Flag the schema upgrade as COMPLETE */
/***************************************/
UPDATE UpgradeLog SET endDate = sysdate, state = 'COMPLETE'
 WHERE release = '2_1_9_5_1';
COMMIT;
